@@ -139,8 +139,8 @@ XML_Saving_Archive::emit_unicode(Unicode uni, int & space, bool & char_mode)
       }
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const UCS_string & ucs)
+void
+XML_Saving_Archive::save_UCS(const UCS_string & ucs)
 {
 int space = do_indent();
 char cc[40];
@@ -155,8 +155,6 @@ bool char_mode = false;
 
    NEED(3, false) << "\"/>" << endl;
    --indent;
-
-   return *this;
 }
 //-----------------------------------------------------------------------------
 XML_Saving_Archive &
@@ -273,8 +271,8 @@ char cc[80];
       }
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const Function & fun)
+void
+XML_Saving_Archive::save_function(const Function & fun)
 {
    do_indent();
    out << "<Function";
@@ -282,17 +280,15 @@ XML_Saving_Archive::operator <<(const Function & fun)
    out << ">" << endl;
    ++indent;
 
-   *this <<  fun.canonical(false);
+   save_UCS(fun.canonical(false));
 
    --indent;
    do_indent();
    out << "</Function>" << endl;
-
-   return *this;
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const Prefix & prefix)
+void
+XML_Saving_Archive::save_prefix(const Prefix & prefix)
 {
    do_indent();
    out << "<Parser assign-pending=\"" << prefix.get_assign_state()
@@ -305,17 +301,16 @@ XML_Saving_Archive::operator <<(const Prefix & prefix)
    loop(s, prefix.size())
       {
         const Token_loc & tloc = prefix.at(s);
-        *this << tloc;
+        save_token_loc(tloc);
       }
    --indent;
 
    do_indent();
    out << "</Parser>" << endl;
-   return *this;
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const SymbolTable & symtab)
+void
+XML_Saving_Archive::save_symtab(const SymbolTable & symtab)
 {
    // collect all symbols in this symbol table.
    //
@@ -354,7 +349,7 @@ Symbol * all_symbols[symbol_count];
             }
 
         const Symbol * sym = all_symbols[idx];
-        *this << *sym;
+        save_symbol(*sym);
 
         all_symbols[idx] = all_symbols[--symbol_count];
       }
@@ -363,17 +358,15 @@ Symbol * all_symbols[symbol_count];
 
    do_indent();
    out << "</SymbolTable>" << endl << endl;
-
-   return *this;
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const StateIndicator & si)
+void
+XML_Saving_Archive::save_SI_entry(const StateIndicator & si)
 {
-     const Executable & exec = *si.get_executable();
+const Executable & exec = *si.get_executable();
 
    do_indent();
-   out << "<SI-entry level=\"" << (si.get_level() - 1)
+   out << "<SI-entry level=\"" << si.get_level()
        << "\" pc=\"" << si.get_PC()
        << "\" line=\"" << exec.get_line(si.get_PC()) << "\"" << flush;
 
@@ -398,7 +391,7 @@ XML_Saving_Archive::operator <<(const StateIndicator & si)
            }
       }
 
-   out <<">" << endl;
+   out <<">" << endl << flush;
 
    ++indent;
    do_indent();
@@ -420,7 +413,7 @@ XML_Saving_Archive::operator <<(const StateIndicator & si)
         case PM_STATEMENT_LIST:
              out << "<Statements>" << endl;
              ++indent;
-             *this << exec.get_text(0);
+             save_UCS(exec.get_text(0));
              --indent;
              do_indent();
              out << "</Statements>" << endl;
@@ -429,7 +422,7 @@ XML_Saving_Archive::operator <<(const StateIndicator & si)
         case PM_EXECUTE:
              out << "<Execute>" << endl;
              ++indent;
-             *this << exec.get_text(0);
+             save_UCS(exec.get_text(0));
              --indent;
              do_indent();
              out << "</Execute>" << endl;
@@ -440,49 +433,31 @@ XML_Saving_Archive::operator <<(const StateIndicator & si)
 
    // print the parser states...
    //
-   *this << si.current_stack;
+   save_prefix(si.current_stack);
 
    --indent;
 
    do_indent();
    out << "</SI-entry>" << endl << endl;
-
-   return *this;
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const Symbol & sym)
+void
+XML_Saving_Archive::save_symbol(const Symbol & sym)
 {
    do_indent();
    out << "<Symbol name=\"" << sym.get_name() << "\" stack-size=\""
        << sym.value_stack_size() << "\">" << endl;
 
    ++indent;
-   loop(v, sym.value_stack_size())  *this << sym[v];
+   loop(v, sym.value_stack_size())  save_vstack_item(sym[v]);
    --indent;
 
    do_indent();
    out << "</Symbol>" << endl << endl;
-
-   return *this;
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const Token & tok)
-{
-char cc[80];
-   snprintf(cc, sizeof(cc), "%X", tok.get_tag());
-
-   do_indent();
-   out << "<Token tag=\"" << cc << "\"";
-   emit_token_val(tok);
-
-   out << ">" << endl;
-   return *this;
-}
-//-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const Token_loc & tloc)
+void
+XML_Saving_Archive::save_token_loc(const Token_loc & tloc)
 {
 char cc[80];
    snprintf(cc, sizeof(cc), "%X", tloc.tok.get_tag());
@@ -493,7 +468,6 @@ char cc[80];
    emit_token_val(tloc.tok);
 
    out << ">" << endl;
-   return *this;
 }
 //-----------------------------------------------------------------------------
 void
@@ -575,8 +549,8 @@ XML_Saving_Archive::emit_token_val(const Token & tok)
       }
 }
 //-----------------------------------------------------------------------------
-XML_Saving_Archive &
-XML_Saving_Archive::operator <<(const ValueStackItem & vsi)
+void
+XML_Saving_Archive::save_vstack_item(const ValueStackItem & vsi)
 {
    switch(vsi.name_class)
       {
@@ -598,7 +572,7 @@ XML_Saving_Archive::operator <<(const ValueStackItem & vsi)
 
         case NC_FUNCTION:
         case NC_OPERATOR:
-             *this << *vsi.sym_val.function;
+             save_function(*vsi.sym_val.function);
              break;
 
         case NC_SHARED_VAR:
@@ -608,8 +582,6 @@ XML_Saving_Archive::operator <<(const ValueStackItem & vsi)
              break;
 
       }
-
-   return *this;
 }
 //-----------------------------------------------------------------------------
 XML_Saving_Archive &
@@ -731,7 +703,7 @@ tm * t;
 "           hour=\""     <<  t->tm_hour
      << "\" minute=\""   <<  t->tm_min
      << "\" second=\""   <<  t->tm_sec
-     << "\" timezone=\"" << Workspace::get_v_quad_TZ().get_offset()
+     << "\" timezone=\"" << Workspace::get_v_Quad_TZ().get_offset()
      << "\">\n" << endl;
 
    ++indent;
@@ -750,7 +722,7 @@ tm * t;
          if (val.is_forever())   continue;   // e.g. ⍞
          if (val.is_marked())    continue;   // stale
 
-         val.clear_marked();
+         val.unmark();
          values.push_back(_val_par(val));
        }
 
@@ -790,12 +762,12 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
 
    // save user defined symbols
    //
-   *this << Workspace::get_symbol_table();
+   save_symtab(Workspace::get_symbol_table());
 
    // save certain system variables
    //
-#define rw_sv_def(x) *this << Workspace::get_v_quad_ ## x();
-#define ro_sv_def(x) *this << Workspace::get_v_quad_ ## x();
+#define rw_sv_def(x) save_symbol(Workspace::get_v_ ## x());
+#define ro_sv_def(x) save_symbol(Workspace::get_v_ ## x());
 #include "SystemVariable.def"
 
    // save state indicator
@@ -814,7 +786,7 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
               {
                 if (si->get_level() == l)
                    {
-                     *this << *si;
+                     save_SI_entry(*si);
                      break;
                    }
               }
@@ -835,15 +807,18 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
    //
    out << "</Workspace>" << endl
        << char(0) << char(0) <<char(0) <<char(0) << endl;
+
+   return *this;
 }
 //=============================================================================
-XML_Loading_Archive::XML_Loading_Archive(const char * filename)
+XML_Loading_Archive::XML_Loading_Archive(const char * _filename)
    : line(1),
      data(0),
      end(0),
      copying(false),
      protection(false),
-     reading_vids(false)
+     reading_vids(false),
+     filename(_filename)
 {
    fd = open(filename, O_RDONLY);
    if (fd == -1)   return;
@@ -1117,10 +1092,13 @@ const char * tz_sign = (tzone < 0) ? "" : "+";
         << "  " << hour << ":" << minute << ":" << second
         << " (GMT" << tz_sign << tzone/3600 << ")" << endl;
 
-const UTF8 * end = wsid;
-   while (*end != '"')   ++end;
+   if (!copying)
+      {
+        const UTF8 * end = wsid;
+        while (*end != '"')   ++end;
 
-   Workspace::set_WS_name(UCS_string(UTF8_string(wsid, end - wsid)));
+        Workspace::set_WS_name(UCS_string(UTF8_string(wsid, end - wsid)));
+      }
 }
 //-----------------------------------------------------------------------------
 void
@@ -1273,7 +1251,7 @@ const Unicode type = UTF8_string::toUni(first, len);
              }
              break;
 
-        default: Q(type) Q(line) DOMAIN_ERROR;
+        default: Q1(type) Q1(line) DOMAIN_ERROR;
       }
 }
 //-----------------------------------------------------------------------------
@@ -1379,19 +1357,21 @@ const unsigned int vid = find_int_attr("vid", false, 10);
 
    if (d != 0)   symbol.push();
 
-   // symbol.assign() can fail, for example if symbol is a system variable.
-   // inform the user, but continue.
+   // some system variables are  saved for troubleshooting purposes, but
+   // should not be loaded...
+   //
+   if (symbol.is_readonly())   return;
+   if (symbol.get_name() == id_name(ID_Quad_NLT))    return;
+   if (symbol.get_name() == id_name(ID_Quad_SYL))    return;
+
    try
       {
         symbol.assign(values[vid], LOC);
       }
    catch (...)
       {
-        if (symbol.get_name() != id_name(ID_QUAD_SYL))
-           {
-             CERR << "*** Could not assign value " << *values[vid]
-                  << "    to variable " << symbol.get_name() << " ***" << endl;
-           }
+        CERR << "*** Could not assign value " << *values[vid]
+             << "    to variable " << symbol.get_name() << " ***" << endl;
       }
 }
 //-----------------------------------------------------------------------------
@@ -1406,7 +1386,7 @@ const int value = find_int_attr("value", false, 10);
 void
 XML_Loading_Archive::read_Function(int d, Symbol & symbol)
 {
-const int native     = find_int_attr("native", true, 10);
+const int native = find_int_attr("native", true, 10);
 
    next_tag(LOC);
    expect_tag("UCS", LOC);
@@ -1417,7 +1397,6 @@ const UTF8 * uni = find_attr("uni", false);
 UCS_string text;
    while (*uni != '"')   read_chars(text, uni);
 
-int err = 0;
    if (native == 1)
       {
         NativeFunction * nfun = NativeFunction::fix(text, symbol.get_name());
@@ -1436,11 +1415,20 @@ int err = 0;
       }
    else
       {
-        UserFunction * ufun = new UserFunction(text, err, false, LOC);
-        Assert(err == -1);
+        int err = 0;
+        UserFunction * fun = UserFunction::fix(text, err, false, LOC, filename);
 
         if (d == 0)   symbol.pop(false);
-        symbol.push_function(ufun);
+        if (fun)
+           {
+             symbol.push_function(fun);
+           }
+        else
+           {
+             CERR << "fix(" << symbol.get_name() << ") failed at line "
+                  << err << " (" << Workspace::more_error() << ")" << endl;
+             symbol.push();
+           }
       }
 }
 //-----------------------------------------------------------------------------
@@ -1534,6 +1522,14 @@ const bool no_copy = (symbol && protection) || ! is_allowed(name_ucs);
 
    loop(d, depth)
       {
+        // for )COPY skip d > 0
+        //
+        if (copying && (d > 0))
+           {
+             skip_to_tag("/Symbol");
+             return;
+           }
+
         next_tag(LOC);
         if      (is_tag("unused-name"))       read_unused_name(d, *symbol);
         else if (is_tag("Variable"))          read_Variable(d, *symbol);
@@ -1562,7 +1558,7 @@ const int levels = find_int_attr("levels", false, 10);
         next_tag(LOC);
         expect_tag("SI-entry", LOC);
 
-        read_SI_entry(l + 1);   // levels count from 1 ... levels
+        read_SI_entry(l);
 
         // the parsers loop eats the terminating /SI-entry
       }
@@ -1670,7 +1666,7 @@ const int lah_high = find_int_attr("lookahead-high", false, 10);
 
 Prefix & parser = si.current_stack;
 
-   parser.set_assign_state((Prefix::Assign_state)ass_state);
+   parser.set_assign_state((Assign_state)ass_state);
    parser.set_lookahead_high(Function_PC(lah_high));
 
    for (;;)
@@ -1762,7 +1758,7 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
              {
                const unsigned int rank = find_int_attr("rank", false, 10);
                char * vids = (char *)find_attr("index", false);
-               IndexExpr & idx = *new IndexExpr(false, LOC);
+               IndexExpr & idx = *new IndexExpr(ASS_none, LOC);
                while (*vids != '"')
                   {
                     if (*vids == ',')   ++vids;
