@@ -102,37 +102,26 @@ const uint32_t nelm = val->element_count();
    //
    if (type == 0)        // packed bit vector
       {
-        uint8_t accu = 0;
-        int bits = 0;
+        int accu = 0;
 
         loop(e, nelm)
            {
+             const int bit = e%8;
              const APL_Integer i = val->get_ravel(e).get_int_value();
              Assert(i == 0 || i == 1);
-
-             accu += accu;
-             accu += i;
-             ++bits;
-             if (bits == 8)
+             if (i)   accu |= 0x80 >> bit;
+             if (bit == 7)
                 {
-                  result.append(Unicode(accu & 0xFF));
-
+                  result.append(Unicode(accu));
                   accu = 0;
-                  bits = 0;
                 }
-
-             if (bits)   // partly filled
-                {
-                  while (bits < 8)
-                    {
-                      accu += accu;
-                      ++bits;
-                    }
-
-                  result.append(Unicode(accu & 0xFF));
-                }
-
            }
+
+        if (nelm % 8)   // partly filled
+           {
+             result.append(Unicode(accu));
+           }
+
       }
    else if (type == 1)   // 4 byte ints vector
       {
@@ -216,36 +205,37 @@ const uint32_t nelm = val->element_count();
         // by the size of each sub-value. I.e.
         //
         //   +------------------+
-        //   | TOP-LEVEL-HEADER |
+        //   | TOP-LEVEL-HEADER |                        16 bytes + shape
         //   +------------------+
-        //   | OFFSET-1         | --------+
+        //   | OFFSET-1         | --------+               4 bytes
         //   +------------------+         |
-        //   | OFFSET-2         | --------|--+
+        //   | OFFSET-2         | --------|--+            4 bytes
         //   +------------------+         |  |
-        //   | ...              |         |  |
+        //   | ...              |         |  |              ...
         //   +------------------+         |  |
-        //   | OFFSET-nelm      | --------|--|--+
+        //   | OFFSET-nelm      | --------|--|--+         4 bytes
         //   +------------------+         |  |  |
-        //   | SUB-HEADER-1     | <-------+  |  |
+        //   | SUB-HEADER-1     | <-------+  |  |        16 bytes + shape
         //   | VALUE-1          |            |  |
         //   +------------------+            |  |
-        //   | SUB-HEADER-2     | <----------+  |
+        //   | SUB-HEADER-2     | <----------+  |        16 bytes + shape
         //   | VALUE-2          |               |
         //   +------------------+               |
-        //   | ...              |               |
+        //   | ...              |               |           ...
         //   +------------------+               |
-        //   | SUB-HEADER-nelm  | <-------------+
+        //   | SUB-HEADER-nelm  | <-------------+        16 bytes + shape
         //   | VALUE-nelm       |
         //   +------------------+
         //
       
         uint32_t offset = 16 + 4*val->get_rank() + 4*nelm;
+        while (offset & 0x0F)   ++offset;
         loop(e, nelm)
            {
-             result.append(Unicode(offset & 0xFF));   offset >>= 8;
-             result.append(Unicode(offset & 0xFF));   offset >>= 8;
-             result.append(Unicode(offset & 0xFF));   offset >>= 8;
-             result.append(Unicode(offset & 0xFF));   offset >>= 8;
+             result.append(Unicode(offset       & 0xFF));
+             result.append(Unicode(offset >>  8 & 0xFF));
+             result.append(Unicode(offset >> 16 & 0xFF));
+             result.append(Unicode(offset >> 24 & 0xFF));
 
              const Cell & cell = val->get_ravel(e);
              if (cell.is_character_cell() || cell.is_numeric())
@@ -268,10 +258,7 @@ const uint32_t nelm = val->element_count();
         // pad top level to 16 bytes. end is aligned to 16 byte so we
         // case use it as reference.
         //
-        while (result.size() & 0x0F)
-           {
-             result.append(Unicode_0);
-           }
+        while (result.size() & 0x0F)   result.append(Unicode_0);
 
         // append sub-values.
         //
@@ -286,7 +273,6 @@ const uint32_t nelm = val->element_count();
                   const int sub_type = sub_val->get_CDR_type();
                   const int sub_len = sub_val->total_size_brutto(sub_type);
                   fill(result, sub_type, sub_len, sub_val);
-
                 }
               else
                 {
@@ -317,23 +303,27 @@ CDR::from_CDR(const CDR_string & cdr, const char * loc)
 /*
 struct tf3_header
     {
-       uint32_t ptr;      // byte reversed
-       uint32_t nb;       // byte reversed
-       uint32_t nelm;     // byte reversed
-       uint8_t  type;
-       uint8_t  rank;
-       uint8_t  fill[2];
-       uint32_t dim[1];   // byte reversed
+       uint32_t ptr;      //  0: byte reversed
+       uint32_t nb;       //  4: byte reversed
+       uint32_t nelm;     //  8: byte reversed
+       uint8_t  type;     // 12:
+       uint8_t  rank;     // 13:
+       uint8_t  fill[2];  // 14:
+       uint32_t dim[1];   // 16: byte reversed
     }
 */
 
 const uint8_t * data = cdr.get_items();
 
-const uint32_t nelm = CDR::get_4_be(data + 8);
+const uint32_t nelm = get_4_be(data + 8);
 const uint32_t vtype = data[12];
 const Rank rank = data[13];
 Shape shape;
-   loop(r, rank)   shape.add_shape_item(CDR::get_4_be(data + 16 + 4*r));
+   loop(r, rank)
+      {
+        const ShapeItem sh = get_4_be(data + 16 + 4*r);
+        shape.add_shape_item(sh);
+      }
 
 Value_P ret(new Value(shape, loc));
 
@@ -343,7 +333,7 @@ const uint8_t * ravel = data + 16 + 4*rank;
       {
         loop(n, nelm)
            {
-             const char bit = ravel[n >> 3] & (1 << (n & 7));
+             const int bit = ravel[n >> 3] & (0x80 >> (n & 7));
              if (bit)   new (&ret->get_ravel(n)) IntCell(1);
              else       new (&ret->get_ravel(n)) IntCell(0);
            }
@@ -408,28 +398,30 @@ const uint8_t * ravel = data + 16 + 4*rank;
                  {
                    if (sub_vtype == 0)        // bit
                       {
-                        new (&ret->get_ravel(n))   IntCell(*sub_ravel & 0x01);
+                        new (&ret->get_ravel(n))
+                            IntCell((*sub_ravel & 0x80) ? 1 : 0);
                         continue;   // next n
                       }
 
                    if (sub_vtype == 1)        // 4 byte int
                       {
-                        new (&ret->get_ravel(n))   IntCell(get_4_be(sub_ravel));
+                        new (&ret->get_ravel(n))
+                            IntCell(*(uint32_t *)(sub_ravel));
                         continue;   // next n
                       }
 
                    if (sub_vtype == 2)        // 8 byte real
                       {
                         new (&ret->get_ravel(n))
-                            FloatCell((APL_Float)(get_8_be(sub_ravel)));
+                            FloatCell(*(const double *)sub_ravel);
                         continue;   // next n
                       }
 
                    if (sub_vtype == 3)        // 16 byte complex
                       {
-                        new (&ret->get_ravel(n))   ComplexCell(
-                                         (APL_Float)(get_8_be(sub_ravel)),
-                                         (APL_Float)(get_8_be(sub_ravel + 8)));
+                        new (&ret->get_ravel(n))
+                            ComplexCell(*(const double *)sub_ravel,
+                                        *(const double *)(sub_ravel + 8));
                         continue;   // next n
                       }
 
@@ -478,9 +470,9 @@ const uint8_t * ravel = data + 16 + 4*rank;
                  }
 
               const uint32_t sub_cdr_len = get_4_be(sub_data + 4);
-              CDR_string sub_cdr(sub_ravel, sub_cdr_len);
+              CDR_string sub_cdr(sub_data, sub_cdr_len);
               Value_P sub_val = from_CDR(sub_cdr, LOC);
-              Assert(sub_val);
+              Assert(!!sub_val);
               new (&ret->get_ravel(n))   PointerCell(sub_val);
             }
       }
