@@ -28,6 +28,7 @@
 #include "PointerCell.hh"
 #include "PrintOperator.hh"
 #include "Value.hh"
+#include "SystemLimits.hh"
 #include "Workspace.hh"
 
 //-----------------------------------------------------------------------------
@@ -160,8 +161,8 @@ Cell::equal(const Cell & other, APL_Float qct) const
 bool
 Cell::is_near_int(APL_Float value, APL_Float qct)
 {
-   if (value >  9223372036854775807LL)   return false;
-   if (value < -9223372036854775807LL)   return false;
+   if (value > LARGE_INT)   return false;
+   if (value < SMALL_INT)   return false;
 
 const double result = nearbyint(value);
 const double diff = value - result;
@@ -246,44 +247,78 @@ const APL_Float qct = Workspace::get_CT();
    //
    if (comp_len == 1)
       {
-        if (ca[0].equal(cb[0], qct))   return ca > cb;
-        return ca[0].greater(cb, ascending);
+        const bool equal = ca[0].equal(cb[0], qct);
+        if (equal)   return ca > cb;
+        const bool result = ca[0].greater(&cb[0], ascending);
+        return result;
       }
 
    loop(c, comp_len)
       {
-         if (ca[c].equal(cb[c], qct))   continue;
-         return ca[c].greater(cb + c, ascending);
+        const bool equal = ca[c].equal(cb[c], qct);
+        if (equal)   continue;
+        const bool result = ca[c].greater(cb + c, ascending);
+        return result;
       }
 
    return ca > cb;   // a and b are equal: sort by position
 }
 //-----------------------------------------------------------------------------
-ostream & operator <<(ostream & out, const Cell & cell)
+ostream & 
+operator <<(ostream & out, const Cell & cell)
 {
 PrintBuffer pb = cell.character_representation(PR_BOXED_GRAPHIC);
-UCS_string ucs(pb, 0);
+UCS_string ucs(pb, 0, Workspace::get_PW());
    return out << ucs << " ";
 }
 //-----------------------------------------------------------------------------
 void
+Cell::bif_equal(Cell * Z, const Cell * A) const
+{
+int eq = 0;   // assume incompatible cell types ( == not equal)
+
+   if (is_character_cell() == A->is_character_cell())   // compatible cell types
+      {
+       eq = compare(*A) == COMP_EQ;
+      }
+
+   new (Z) IntCell(eq);
+}
+//-----------------------------------------------------------------------------
+void
 Cell::bif_not_equal(Cell * Z, const Cell * A) const
 {
-   bif_equal(Z, A);
-   Z->value.ival = 1 - Z->value.ival;
+int neq = 1;   // assume incompatible cell types ( == not equal)
+
+   if (is_character_cell() == A->is_character_cell())   // compatible cell types
+      {
+       neq = compare(*A) != COMP_EQ;
+      }
+
+   new (Z) IntCell(neq);
 }
 //-----------------------------------------------------------------------------
 void
-Cell::bif_greater_eq(Cell * Z, const Cell * A) const
+Cell::bif_greater_than(Cell * Z, const Cell * A) const
 {
-   bif_equal(Z, A);
-   if (!Z->value.ival)   bif_greater_than(Z, A);
+   new (Z) IntCell((A->compare(*this) == COMP_GT) ? 1 : 0);
 }
 //-----------------------------------------------------------------------------
 void
 Cell::bif_less_eq(Cell * Z, const Cell * A) const
 {
-   bif_equal(Z, A);
-   if (!Z->value.ival)   bif_less_than(Z, A);
+   new (Z) IntCell((A->compare(*this) != COMP_GT) ? 1 : 0);
+}
+//-----------------------------------------------------------------------------
+void
+Cell::bif_less_than(Cell * Z, const Cell * A) const
+{
+   new (Z) IntCell((A->compare(*this) == COMP_LT) ? 1 : 0);
+}
+//-----------------------------------------------------------------------------
+void
+Cell::bif_greater_eq(Cell * Z, const Cell * A) const
+{
+   new (Z) IntCell((A->compare(*this) != COMP_LT) ? 1 : 0);
 }
 //-----------------------------------------------------------------------------
