@@ -51,6 +51,8 @@
 #define PATH_MAX 4096
 #endif
 
+int Command::boxing_format = -1;
+
 //-----------------------------------------------------------------------------
 void
 Command::process_line()
@@ -80,6 +82,13 @@ Command::process_line(UCS_string & line)
                 vector<UCS_string> args;
                 arg.copy_black_list(args);
                 line.clear();
+                if (!cmd.starts_iwith(")MORE")) 
+                   {
+                     // clear )MORE info unless command is )MORE
+                     //
+                     Workspace::more_error().clear();
+                   }
+
 #define cmd_def(cmd_str, code, _arg) \
                 if (cmd.starts_iwith(cmd_str)) { code } else
 #include "Command.def"
@@ -108,6 +117,8 @@ Command::process_line(UCS_string & line)
 
    // at this point, line should be a statement list. Parse it...
    //
+   Workspace::more_error().clear();
+
 const Executable * statements = 0;
    try
       {
@@ -242,7 +253,7 @@ const Executable * statements = 0;
               if (si == 0)
                  {
                     Workspace::more_error() = UCS_string(
-                          _("empty branch (→⍳0) without suspended function"));
+                          _("branch back into function (→N) without suspended function"));
                     SYNTAX_ERROR;   // →N without function,
                  }
 
@@ -297,7 +308,7 @@ const Executable * statements = 0;
 
          // we should not come here.
          //
-         Q(token)  Q(token.get_Class())  Q(token.get_tag())  FIXME;
+         Q1(token)  Q1(token.get_Class())  Q1(token.get_tag())  FIXME;
        }
 
    // pop the context for the statements
@@ -306,6 +317,30 @@ const Executable * statements = 0;
 }
 //-----------------------------------------------------------------------------
 void 
+Command::cmd_BOXING(ostream & out, const UCS_string & arg)
+{
+int format = arg.atoi();
+
+   if (arg.starts_iwith("OFF"))   format = -1;
+
+   switch (format)
+      {
+        case -1:
+        case  2:
+        case  3:
+        case  4:
+        case  7:
+        case  8:
+        case  9: boxing_format = format;
+                 return;
+
+        default:   out << "Bad ]BOXING parameter " << arg
+                       << " (valid values are: OFF, 2, 3, 4, 7, 8, and 9)"
+                       << endl;
+      }
+}
+//-----------------------------------------------------------------------------
+void 
 Command::cmd_CHECK(ostream & out)
 {
    // erase stale functions from failed ⎕EX
@@ -441,6 +476,12 @@ Command::cmd_HELP(ostream & out)
 void
 Command::cmd_HOST(ostream & out, const UCS_string & arg)
 {
+   if (safe_mode)
+      {
+        out << _(")HOST command not allowed in safe mode.") << endl;
+        return;
+      }
+
 UTF8_string host_cmd(arg);
 FILE * pipe = popen(host_cmd.c_str(), "r");
    if (pipe == 0)   // popen failed
@@ -974,8 +1015,7 @@ const ShapeItem ec = val->element_count();
 }
 //-----------------------------------------------------------------------------
 void
-Command::transfer_context::array_2TF(const vector<UCS_string> &
-                                        objects) const
+Command::transfer_context::array_2TF(const vector<UCS_string> & objects) const
 {
    // an Array in 2 ⎕TF format
    //
@@ -997,7 +1037,7 @@ UCS_string var_or_fun;
         if (!var_name.contained_in(objects))   return;
       }
 
-   Quad_TF::tf2_parse(data1, var_or_fun);
+   var_or_fun = Quad_TF::tf2_inv(data1);
 
    Assert(var_or_fun.size());
 }
@@ -1020,8 +1060,7 @@ UCS_string statement;
    while (idx < data.size())   statement.append(data[idx++]);
    statement.append(UNI_ASCII_LF);
 
-UCS_string fun_name1;
-   Quad_TF::tf2_parse(statement, fun_name1);
+UCS_string fun_name1 = Quad_TF::tf2_inv(statement);
 
 Symbol * sym1 = Workspace::lookup_existing_symbol(fun_name1);
    Assert(sym1);
