@@ -173,6 +173,8 @@ ComplexCell::equal(const Cell & A, APL_Float qct) const
 {
    if (!A.is_numeric())   return false;
 
+   // real part
+   //
    {
      const APL_Float valA = A.get_real_value();
      const APL_Float valB =   get_real_value();
@@ -181,10 +183,12 @@ ComplexCell::equal(const Cell & A, APL_Float qct) const
      const APL_Float maxAB = posA > posB ? posA : posB;
      const APL_Float maxCT = qct * maxAB;
 
-     if (valB < (valA - maxCT))   return false;
-     if (valB > (valA + maxCT))   return false;
+     if (posB < (posA - maxCT))   return false;
+     if (posB > (posA + maxCT))   return false;
    }
 
+   // imag part
+   //
    {
      const APL_Float valA = A.get_imag_value();
      const APL_Float valB =   get_imag_value();
@@ -193,8 +197,8 @@ ComplexCell::equal(const Cell & A, APL_Float qct) const
      const APL_Float maxAB = posA > posB ? posA : posB;
      const APL_Float maxCT = qct * maxAB;
 
-     if (valB < (valA - maxCT))   return false;
-     if (valB > (valA + maxCT))   return false;
+     if (posB < (posA - maxCT))   return false;
+     if (posB > (posA + maxCT))   return false;
    }
 
    return true;
@@ -203,7 +207,21 @@ ComplexCell::equal(const Cell & A, APL_Float qct) const
 bool
 ComplexCell::greater(const Cell * other, bool ascending) const
 {
-   DOMAIN_ERROR;
+   switch(other->get_cell_type())
+      {
+        case CT_CHAR:    return ascending;
+        case CT_INT:
+        case CT_FLOAT:
+        case CT_COMPLEX: break;
+        case CT_POINTER: return !ascending;
+        case CT_CELLREF: DOMAIN_ERROR;
+        defaulkt:        Assert(0 && "Bad celltype");
+      }
+
+const Comp_result comp = compare(*other);
+   if (comp == COMP_EQ)   return this > other;
+   if (comp == COMP_GT)   return ascending;
+   else                   return !ascending;
 }
 //-----------------------------------------------------------------------------
 void
@@ -213,6 +231,48 @@ ComplexCell::release(const char * loc)
    new (this) Cell;
 }
 //-----------------------------------------------------------------------------
+Comp_result
+ComplexCell::compare(const Cell & other) const
+{
+   // comparison of complex numbers gives DOMAN ERROR if one of the cells
+   // is not near-real.
+   //
+const APL_Float qct = Workspace::get_CT();
+
+   if (get_imag_value() < -qct)   DOMAIN_ERROR;   // not near-real
+   if (get_imag_value() >  qct)   DOMAIN_ERROR;   // not near-real
+
+const APL_Float rval = get_real_value();
+   if (other.is_integer_cell())   // integer
+      {
+        if (equal(other, qct))   return COMP_EQ;
+        return (rval < other.get_int_value()) ? COMP_LT : COMP_GT;
+      }
+
+   if (other.is_float_cell())
+      {
+        if (equal(other, qct))   return COMP_EQ;
+        return (rval < other.get_real_value()) ? COMP_LT : COMP_GT;
+      }
+
+   if (other.is_complex_cell())   // complex
+      {
+        if (other.get_imag_value() < -qct)   DOMAIN_ERROR;   // not near-real
+        if (other.get_imag_value() >  qct)   DOMAIN_ERROR;   // not near-real
+
+        if (equal(other, qct))   return COMP_EQ;
+        return (rval < other.get_real_value()) ? COMP_LT : COMP_GT;
+        return COMP_EQ;
+      }
+
+   if (other.is_character_cell())
+      {
+        return COMP_GT;
+      }
+
+   DOMAIN_ERROR;
+}
+//-----------------------------------------------------------------------------
 // monadic build-in functions...
 //-----------------------------------------------------------------------------
 void
@@ -381,17 +441,100 @@ const APL_Float qct = Workspace::get_CT();
       }
 }
 //-----------------------------------------------------------------------------
-void ComplexCell::bif_equal(Cell * Z, const Cell * A) const
+void
+ComplexCell::bif_maximum(Cell * Z, const Cell * A) const
 {
+   // maximum of complex numbers gives DOMAN ERROR if one of the cells
+   // is not near-real.
+   //
+const APL_Float qct = Workspace::get_CT();
+
+   if (get_imag_value() < -qct)   DOMAIN_ERROR;   // not near-real
+   if (get_imag_value() >  qct)   DOMAIN_ERROR;   // not near-real
+
+const APL_Float rval = get_real_value();
+
+   if (A->is_integer_cell())
+      {
+         const APL_Integer a = A->get_int_value();
+         if (APL_Float(a) >= rval)   new (Z) IntCell(a);
+         else                        new (Z) FloatCell(rval);
+      }
+   else if (A->is_float_cell())
+      {
+         const APL_Float a = A->get_real_value();
+         if (a >= rval)   new (Z) FloatCell(a);
+         else             new (Z) FloatCell(rval);
+      }
+   else
+      {
+        if (A->get_imag_value() < -qct)   DOMAIN_ERROR;   // not near-real
+        if (A->get_imag_value() >  qct)   DOMAIN_ERROR;   // not near-real
+
+         const APL_Float a = A->get_real_value();
+         if (a >= rval)   new (Z) FloatCell(a);
+         else             new (Z) FloatCell(rval);
+      }
+
+}
+//-----------------------------------------------------------------------------
+void
+ComplexCell::bif_minimum(Cell * Z, const Cell * A) const
+{
+   // minimum of complex numbers gives DOMAN ERROR if one of the cells
+   // is not near real.
+   //
+const APL_Float qct = Workspace::get_CT();
+
+   if (get_imag_value() < -qct)   DOMAIN_ERROR;   // not near-real
+   if (get_imag_value() >  qct)   DOMAIN_ERROR;   // not near-real
+
+const APL_Float rval = get_real_value();
+
+   if (A->is_integer_cell())
+      {
+         const APL_Integer a = A->get_int_value();
+         if (APL_Float(a) <= rval)   new (Z) IntCell(a);
+         else                        new (Z) FloatCell(rval);
+      }
+   else if (A->is_float_cell())
+      {
+         const APL_Float a = A->get_real_value();
+         if (a <= rval)   new (Z) FloatCell(a);
+         else             new (Z) FloatCell(rval);
+      }
+   else
+      {
+        if (A->get_imag_value() < -qct)   DOMAIN_ERROR;   // not near-real
+        if (A->get_imag_value() >  qct)   DOMAIN_ERROR;   // not near-real
+
+         const APL_Float a = A->get_real_value();
+         if (a <= rval)   new (Z) FloatCell(a);
+         else             new (Z) FloatCell(rval);
+      }
+}
+//-----------------------------------------------------------------------------
+void
+ComplexCell::bif_equal(Cell * Z, const Cell * A) const
+{
+const APL_Float qct = Workspace::get_CT();
    if (A->is_complex_cell())
       {
-        const APL_Float qct = Workspace::get_CT();
-        const APL_Complex diff = A->get_complex_value() - *value.cpxp;
-        if      (diff.real() >  qct)   new (Z) IntCell(0);
-        else if (diff.real() < -qct)   new (Z) IntCell(0);
-        else if (diff.imag() >  qct)   new (Z) IntCell(0);
-        else if (diff.imag() > -qct)   new (Z) IntCell(0);
-        else                           new (Z) IntCell(1);
+        const APL_Complex diff = A->get_complex_value() - get_complex_value();
+        if      (diff.real()      >  qct)   new (Z) IntCell(0);
+        else if (diff.real()      < -qct)   new (Z) IntCell(0);
+        else if (diff.imag()      >  qct)   new (Z) IntCell(0);
+        else if (diff.imag()      < -qct)   new (Z) IntCell(0);
+        else                                new (Z) IntCell(1);
+      }
+   else if (A->is_numeric())
+      {
+        const APL_Float diff = A->get_real_value() - get_real_value();
+        if      (diff             >  qct)   new (Z) IntCell(0);
+        else if (diff             < -qct)   new (Z) IntCell(0);
+        else if (get_imag_value() >  qct)   new (Z) IntCell(0);
+        else if (get_imag_value() < -qct)   new (Z) IntCell(0);
+        else                                new (Z) IntCell(1);
       }
    else
       {
@@ -399,7 +542,8 @@ void ComplexCell::bif_equal(Cell * Z, const Cell * A) const
       }
 }
 //-----------------------------------------------------------------------------
-void ComplexCell::bif_power(Cell * Z, const Cell * A) const
+void
+ComplexCell::bif_power(Cell * Z, const Cell * A) const
 {
    if (A->is_complex_cell())
       {
@@ -407,6 +551,13 @@ void ComplexCell::bif_power(Cell * Z, const Cell * A) const
         new (Z) ComplexCell(z);
         Z->demote_complex_to_real(Workspace::get_CT());
       }
+   else if (A->is_numeric())   // real to the power of complex
+      {
+        APL_Complex complex_A(A->get_real_value(), 0);
+        APL_Complex z = pow(complex_A, get_complex_value());
+        new (Z) ComplexCell(z);
+        Z->demote_complex_to_real(Workspace::get_CT());
+      }
    else
       {
         DOMAIN_ERROR;
