@@ -50,6 +50,14 @@ UCS_string line;
    if (line.size())   text.push_back(line);
 }
 //-----------------------------------------------------------------------------
+Executable::Executable(Fun_signature sig, const UCS_string & fname,
+           const UCS_string & lambda_text, const char * loc)
+   : alloc_loc(loc)
+{
+   text.push_back(UserFunction_header::lambda_header(sig, fname));
+   text.push_back(lambda_text);
+}
+//-----------------------------------------------------------------------------
 Executable::~Executable()
 {
    Log(LOG_UserFunction__fix)
@@ -69,12 +77,19 @@ Executable::parse_body_line(Function_Line line, const UCS_string & ucs_line,
       CERR << "[" << line << "]" << ucs_line << endl;
 
 Token_string in;
-Parser parser(get_parse_mode(), loc);
+const Parser parser(get_parse_mode(), loc);
    {
      ErrorCode ec = parser.parse(ucs_line, in);
      if (ec != E_NO_ERROR)   throw_parse_error(ec, LOC, LOC);
-   } 
+   }
 
+   parse_body_line(line, in, loc);
+} 
+//-----------------------------------------------------------------------------
+void
+Executable::parse_body_line(Function_Line line, const Token_string & in,
+                            const char * loc)
+{
 Source<Token> src(in);
 
    // handle labs (if any)
@@ -95,54 +110,79 @@ Source<Token> src(in);
 Token_string out;
    while (src.rest())
       {
-        uint32_t stat_len = src.rest();   // assume statement == line
-
-        // find next diamond. If there is one, adjust stat_len accordingly.
+        // find next diamond. If there is one, copy statement in reverse order,
+        // append TOK_ENDL, and continue.
         //
-        loop(t, src.rest())
-            {
+        {
+          uint32_t stat_len = 0;   // statement length, not counting ◊
+          bool got_diamond = false;
+          loop(t, src.rest())
+              {
               if (src[t].get_tag() == TOK_DIAMOND)
                  {
-                   stat_len = t;
+                   got_diamond = true;
                    break;
                  }
-            }
-
-        Source<Token> statement(src, 0, stat_len);
-        src.skip(stat_len + 1);
-
-        // copy in reverse order
-        //
-        while (statement.rest())
-            {
-              // check for invalid token.
-              //
-              const Token & tok = statement.get_last();
-              if ((tok.get_tag() & TC_MASK) >= TC_MAX_PERM)
+              else
                  {
-                   CERR << "Line " << line << endl
-                        << "Offending token: (tag > TC_MAX_PERM) "
-                        << tok.get_tag() << " " << tok << endl
-                        << "Statement: ";
-                   Token::print_token_list(CERR, statement);
-                   SYNTAX_ERROR;
+                   ++stat_len;
                  }
+              }
 
-               out.append(Token());
-               move_2(out[out.size() - 1], tok, LOC);
-            }
+          if (got_diamond)
+             {
+               loop(s, stat_len)
+                   {
+                     const Token & tok = src[stat_len - s - 1];
+                     if (tok.get_Class() >= TC_MAX_PERM &&
+                         tok.get_tag() != TOK_L_CURLY &&
+                         tok.get_tag() != TOK_R_CURLY)
+                        {
+                          CERR << "Line " << line << endl
+                               << "Offending token: (tag > TC_MAX_PERM) "
+                               << tok.get_tag() << " " << tok << endl
+                               << "Statement: ";
+                          Token::print_token_list(CERR, in);
+                          SYNTAX_ERROR;
+                        }
+                     out.append(tok);
+                   }
+               if (stat_len)   out.append(Token(TOK_ENDL), LOC);
+               src.skip(stat_len + 1);
+               continue;
+           }
+        }
 
-
-        // add an appropriate end token.
+        // at this point src contains only the last statement.
+        // copy it in reverse order.
         //
-        if (get_parse_mode() == PM_EXECUTE)   ;
-        else if (out.size())                  out.append(Token(TOK_ENDL), LOC);
+        loop(s, src.rest())
+           {
+             const Token & tok = src[src.rest() - s - 1];
+             if (tok.get_Class() >= TC_MAX_PERM &&
+                 tok.get_tag()   != TOK_L_CURLY &&
+                 tok.get_tag()   != TOK_R_CURLY)
+                {
+                  CERR << "Line " << line << endl
+                       << "Offending token: (tag > TC_MAX_PERM) "
+                       << tok.get_tag() << " " << tok << endl
+                       << "Statement: ";
+                  Token::print_token_list(CERR, in);
+                  SYNTAX_ERROR;
+                }
+             out.append(tok);
+           }
+        src.skip(src.rest());
+
+        // add an appropriate end token
+        //
+        if (get_parse_mode() != PM_EXECUTE)   out.append(Token(TOK_ENDL), LOC);
       }
 
    Log(LOG_UserFunction__set_line)
       {
         CERR << "[non-reverse " << line << "] ";
-        parser.print_token_list(CERR, out, 0);
+        Parser::print_token_list(CERR, out, 0);
       } 
 
    loop(t, out.size())   body.append(out[t], LOC);
@@ -233,8 +273,8 @@ UCS_string & message_2 = error.error_message_2;
 
    Log(LOG_prefix__location_info)
       {
-        Q(pc_from_to.low)
-        Q(pc_from_to.high)
+        Q1(pc_from_to.low)
+        Q1(pc_from_to.high)
       }
 
    if (body[pc_from_to.high].get_Class() == TC_RETURN &&
@@ -276,10 +316,10 @@ Function_PC end = start;
 
    Log(LOG_prefix__location_info)
       {
-        Q(start)
-        Q(pc_from_to.low)
-        Q(pc_from_to.high)
-        Q(end)
+        Q1(start)
+        Q1(pc_from_to.low)
+        Q1(pc_from_to.high)
+        Q1(end)
       }
 
    // Line 2: statement
@@ -347,7 +387,7 @@ Executable::unmark_all_values() const
         if (tok.get_ValueType() != TV_VAL)      continue;
 
         Value_P value = tok.get_apl_val();
-        if (!!value)   value->clear_marked();
+        if (!!value)   value->unmark();
       }
 }
 //-----------------------------------------------------------------------------
@@ -370,6 +410,194 @@ int count = 0;
 
    return count;
 }
+//-----------------------------------------------------------------------------
+void
+Executable::setup_lambdas()
+{
+   loop(b, body.size())
+       {
+         // body is in reverse order, so } comes before { (and optional ← NAME)
+         //
+         if (body[b].get_tag() != TOK_R_CURLY)   continue;   // not {
+
+         // a named lambda has the form Q←{ ... }
+         //
+         // however, eg. Q←{ ... } / ... is not a named lambda by an unnamed
+         // lambda for / whose result is assigned to Q. We need to distinguish
+         // the two cases and remember if the token after } is the end of
+         // the statement or not.
+         //
+         const bool maybe_named = b == 0 || (body[b-1].get_tag() == TOK_DIAMOND
+                                         || body[b-1].get_tag() == TOK_ENDL);
+
+         body[b++].clear(LOC);   // invalidate }
+         Token_string rev_lambda_body;
+
+         int signature = SIG_NONE;
+         int body_nesting = 1;   /// the TOK_R_CURLY above
+
+         for (bool goon = true; goon;)
+             {
+               if (b >= body.size())   SYNTAX_ERROR;
+               Token t;
+               move_1(t, body[b], LOC);
+               body[b].clear(LOC);   // invalidate in main body
+
+               switch(t.get_tag())
+                  {
+                    case TOK_L_CURLY:
+                         if (body_nesting == 1)  { goon = false;   continue; }
+                         else               { --body_nesting;      break;    }
+
+                    case TOK_ALPHA:    signature |= SIG_A;    /* no break */
+                    case TOK_OMEGA:    signature |= SIG_B;    break;
+                    case TOK_CHI:      signature |= SIG_X;    break;
+                    case TOK_OMEGA_U:  signature |= SIG_RO;   /* no break */
+                    case TOK_ALPHA_U:  signature |= SIG_LO;   break;
+
+                    case TOK_R_CURLY:  ++body_nesting;        break;
+                    case TOK_DIAMOND:  DEFN_ERROR;
+                    case TOK_BRANCH:   DEFN_ERROR;
+                    case TOK_ESCAPE:   DEFN_ERROR;
+                  }
+
+               rev_lambda_body.append(t, LOC);
+               ++b;
+             }
+
+         // if the lambda is { } then we can't assign anything to λ
+         // and make the lambda result-less.
+         //
+         if (rev_lambda_body.size())   signature |= SIG_Z;
+
+         Token_string forw_lambda_body;
+         if (signature & SIG_Z)
+              {
+                Token ret_lambda(TOK_RETURN_SYMBOL, &Workspace::get_v_LAMBDA());
+                forw_lambda_body.append(ret_lambda, LOC);
+
+                forw_lambda_body.append(Token(TOK_ENDL));
+
+                Symbol * sym_Z = &Workspace::get_v_LAMBDA();
+                forw_lambda_body.append(Token(TOK_LAMBDA, sym_Z), LOC);
+                forw_lambda_body.append(Token(TOK_ASSIGN), LOC);
+
+                const ShapeItem body_len = rev_lambda_body.size();
+                loop(r, body_len)
+                    forw_lambda_body.append(
+                                    rev_lambda_body[body_len - r - 1]);
+              }
+           else
+              {
+                Token ret_void(TOK_RETURN_VOID);
+                forw_lambda_body.append(ret_void, LOC);
+              }
+
+
+         // at this point {} was copied from body to forw_lambda_body and
+         // b is at the (now invalidated) { token.
+         // check if lambda is named, i.e. Name ← { ... }
+         //
+         UCS_string lambda_name;
+         Symbol * lambda_sym = 0;
+         if (   maybe_named && (b + 2) < body.size()
+             && body[b + 1].get_tag() == TOK_ASSIGN
+             && body[b + 2].get_tag() == TOK_LSYMB)
+            {
+              // named lambda
+              //
+              lambda_sym = body[b + 2].get_sym_ptr();
+              lambda_name = lambda_sym->get_name();
+              signature |= SIG_FUN;
+
+               body[b + 1].clear(LOC);
+               body[b + 2].clear(LOC);
+            }
+         else
+            {
+              lambda_name.append(UNI_LAMBDA);
+              lambda_name.append_number(unnamed_lambdas.size() + 1);
+            }
+
+         UCS_string lambda_text;
+         if (signature & SIG_Z)   lambda_text.append_utf8("λ←");
+
+         int lambda_skip = unnamed_lambdas.size() + named_lambdas.size();
+         int opening_curly = 0;
+         int closing_curly = 0;
+         int tidx = 0;
+         int tcol = 0;
+
+         for (;;)
+             {
+               // search for the lambda_skip'th { in the body text
+               //
+               Assert(tidx < text.size());
+               const UCS_string & line = text[tidx];
+
+               if (tcol >= line.size())   // end of line: wrap to next line
+                  {
+                    ++tidx;
+                    tcol = 0;
+                    continue;
+                  }
+
+               {
+                 const Unicode uni = line[tcol++];
+                 if      (uni == UNI_ASCII_L_CURLY)   ++opening_curly;
+                 else if (uni == UNI_ASCII_R_CURLY)   ++closing_curly;
+
+                 if (uni != UNI_ASCII_L_CURLY)   continue;   // not {
+               }
+
+               // found { (but maybe from a previous lambda
+               // or from a sub-lambds)
+               //
+               if (lambda_skip)   { --lambda_skip;   continue; }
+
+               if (tcol >= line.size())   SYNTAX_ERROR;
+               for (;;)
+                   {
+                     const Unicode uni = line[tcol++];
+                     if      (uni == UNI_ASCII_L_CURLY)   ++opening_curly;
+                     else if (uni == UNI_ASCII_R_CURLY)   ++closing_curly;
+
+                     // if uni was } then it could be:
+                     //
+                     // 1. a sub-{}      as in { {...} or
+                     // 2. a top-level-} as in { ,,, }
+                     // 
+                     // opening_curly == closing_curly distinguishes 1. and 2.
+                     //
+                     if (opening_curly == closing_curly)   break;
+                     lambda_text.append(uni);
+                   }
+               break;
+             }
+
+         UserFunction * ufun = new UserFunction(Fun_signature(signature),
+                                                lambda_name, lambda_text,
+                                                forw_lambda_body);
+
+         if (signature & SIG_FUN)   // named lambda
+            {
+              named_lambdas.push_back(ufun);
+
+              // replace the lambda by its name, but as committed value
+              // so that it doesn't print (similar to ⎕FX).
+              //
+              Value_P val_name(new Value(lambda_name, LOC));
+              move_2(body[b], Token(TOK_APL_VALUE2, val_name), LOC);
+            }
+         else                       // unnamed lambda
+            {
+              unnamed_lambdas.push_back(ufun);
+              move_2(body[b], ufun->get_token(), LOC);
+            }
+       }
+
+   Parser::remove_void_token(body);
+}
 //=============================================================================
 ExecuteList *
 ExecuteList::fix(const UCS_string & data, const char * loc)
@@ -447,6 +675,7 @@ StatementList * fun = new StatementList(data, loc);
    }
 
    fun->parse_body_line(Function_Line_0, data, loc);
+   fun->setup_lambdas();
 
    Log(LOG_UserFunction__fix)
       {
