@@ -25,12 +25,12 @@
 #include "Workspace.hh"
 
 //-----------------------------------------------------------------------------
-IndexExpr::IndexExpr(bool _left, const char * loc)
+IndexExpr::IndexExpr(Assign_state astate, const char * loc)
    : DynamicObject(loc),
      quad_ct(Workspace::get_CT()),
      quad_io(Workspace::get_IO()),
      rank(0),
-     left(_left)
+     assign_state(astate)
 {
 }
 //-----------------------------------------------------------------------------
@@ -45,6 +45,35 @@ Value_P ret = values[rk];
    return ret;
 }
 //-----------------------------------------------------------------------------
+void
+IndexExpr::extract_all()
+{
+   loop(r, rank)
+      {
+        Value_P val = extract_value(r);
+      }
+}
+//-----------------------------------------------------------------------------
+bool
+IndexExpr::check_range(const Shape & shape) const
+{
+   loop(r, rank)
+      {
+        Value_P ival = values[rank - r - 1];
+        if (!ival)   continue;   // elided index
+
+        const ShapeItem max_idx = shape.get_shape_item(r) + quad_io;
+        loop(i, ival->element_count())
+           {
+             const APL_Integer idx = ival->get_ravel(i).get_near_int(quad_ct);
+             if (idx < quad_io)    return true;
+             if (idx >= max_idx)   return true;
+           }
+      }
+
+   return false;
+}
+//-----------------------------------------------------------------------------
 Rank
 IndexExpr::get_axis(Rank max_axis) const
 {
@@ -158,7 +187,7 @@ operator <<(ostream & out, const IndexExpr & idx)
              pctx.set_style(PR_APL_MIN);
              PrintBuffer pb(*idx.values[i], pctx);
 
-             UCS_string ucs(pb, idx.values[i]->get_rank());
+             UCS_string ucs(pb, idx.values[i]->get_rank(), Workspace::get_PW());
              out << ucs;
            }
       }
