@@ -43,21 +43,10 @@
 bool
 IntCell::greater(const Cell * other, bool ascending) const
 {
-const uint64_t this_val  = get_int_value();
+const APL_Integer this_val  = get_int_value();
 
    switch(other->get_cell_type())
       {
-        case CT_NONE:
-        case CT_BASE:
-             Assert(0);
-
-        case CT_CHAR:
-             {
-               const Unicode other_val = other->get_char_value();
-               if (this_val == other_val)   return !ascending;
-               return this_val > other_val ? ascending : !ascending;
-             }
-
         case CT_INT:
              {
                const APL_Integer other_val = other->get_int_value();
@@ -68,17 +57,21 @@ const uint64_t this_val  = get_int_value();
         case CT_FLOAT:
              {
                const APL_Float other_val = other->get_real_value();
-               if (this_val == other_val)   return true;
+               if (this_val == other_val)   return this > other;
                return this_val > other_val ? ascending : !ascending;
              }
 
-        case CT_COMPLEX:
-        case CT_POINTER:
-             DOMAIN_ERROR;
-
-        default:
-           Assert(0);
+        case CT_COMPLEX: break;
+        case CT_CHAR:    return ascending;   // never greater
+        case CT_POINTER: return !ascending;
+        case CT_CELLREF: DOMAIN_ERROR;
+        defaulkt:        Assert(0 && "Bad celltype");
       }
+
+const Comp_result comp = compare(*other);
+   if (comp == COMP_EQ)   return this > other;
+   if (comp == COMP_GT)   return ascending;
+   else                   return !ascending;
 }
 //-----------------------------------------------------------------------------
 bool
@@ -89,6 +82,23 @@ IntCell::equal(const Cell & other, APL_Float qct) const
    return other.equal(*this, qct);
 }
 //-----------------------------------------------------------------------------
+Comp_result
+IntCell::compare(const Cell & other) const
+{
+   if (other.is_integer_cell())
+      {
+       if (value.ival == other.get_int_value())   return COMP_EQ;
+       return (value.ival < other.get_int_value()) ? COMP_LT : COMP_GT;
+      }
+
+   if (other.is_numeric())   // float or complex
+      {
+        return (Comp_result)(-other.compare(*this));
+      }
+
+   DOMAIN_ERROR;
+}
+//-----------------------------------------------------------------------------
 // monadic built-in functions...
 //-----------------------------------------------------------------------------
 void
@@ -163,8 +173,8 @@ void IntCell::bif_direction(Cell * Z) const
 //-----------------------------------------------------------------------------
 void IntCell::bif_magnitude(Cell * Z) const
 {
-   if (value.ival >= 0)   new (Z) IntCell( value.ival);
-   else                   new (Z) IntCell(-value.ival);
+   if (value.ival >= 0)   new (Z) IntCell(value.ival);
+   else                   bif_negative(Z);
 }
 //-----------------------------------------------------------------------------
 void IntCell::bif_reciprocal(Cell * Z) const
@@ -178,22 +188,26 @@ void IntCell::bif_reciprocal(Cell * Z) const
       }
 }
 //-----------------------------------------------------------------------------
-void IntCell::bif_roll(Cell * Z) const
+void
+IntCell::bif_roll(Cell * Z) const
 {
 const APL_Integer qio = Workspace::get_IO();
 const APL_Integer set_size = value.ival;
-   if (set_size <= 0)   DOMAIN_ERROR;
+   if (set_size < 0)   DOMAIN_ERROR;
 
 const APL_Integer rnd = Workspace::get_RL();
-   new (Z) IntCell(qio + (rnd % set_size));
+   if (set_size == 0)   new (Z) FloatCell((1.0*rnd)/0x8000000000000000ULL);
+   else                 new (Z) IntCell(qio + (rnd % set_size));
 }
 //-----------------------------------------------------------------------------
-void IntCell::bif_pi_times(Cell * Z) const
+void
+IntCell::bif_pi_times(Cell * Z) const
 {
    new (Z) FloatCell(M_PI * value.ival);
 }
 //-----------------------------------------------------------------------------
-void IntCell::bif_ceiling(Cell * Z) const
+void
+IntCell::bif_ceiling(Cell * Z) const
 {
    new (Z) IntCell(value.ival);
 }
@@ -208,7 +222,8 @@ void IntCell::bif_exponential(Cell * Z) const
    new (Z) FloatCell(exp(value.ival));
 }
 //-----------------------------------------------------------------------------
-void IntCell::bif_nat_log(Cell * Z) const
+void
+IntCell::bif_nat_log(Cell * Z) const
 {
    if (value.ival > 0)
       {
@@ -228,6 +243,135 @@ void IntCell::bif_nat_log(Cell * Z) const
 //-----------------------------------------------------------------------------
 // dyadic built-in functions...
 //-----------------------------------------------------------------------------
+void
+IntCell::bif_add(Cell * Z, const Cell * A) const
+{
+   if (!A->is_integer_cell())
+      {
+        A->bif_add(Z, this);
+        return;
+      }
+
+   // both cells are integers.
+   //
+const int64_t a = A->get_int_value();
+const int64_t b =    get_int_value();
+const double sum = a + (double)b;
+
+   if (sum > LARGE_INT || sum < SMALL_INT)   new (Z) FloatCell(sum);
+   else                                      new (Z) IntCell(a + b);
+}
+//-----------------------------------------------------------------------------
+void
+IntCell::bif_subtract(Cell * Z, const Cell * A) const
+{
+   if (!A->is_integer_cell())
+      {
+        this->bif_negative(Z);   // Z = -B
+        A->bif_add(Z, Z);        // Z = A + Z = A + -B
+        return;
+      }
+
+   // both cells are integers.
+   //
+const int64_t a = A->get_int_value();
+const int64_t b =    get_int_value();
+const double diff = a - (double)b;
+
+   if (diff > LARGE_INT || diff < SMALL_INT)   new (Z) FloatCell(diff);
+   else                                        new (Z) IntCell(a - b);
+}
+//-----------------------------------------------------------------------------
+void
+IntCell::bif_divide(Cell * Z, const Cell * A) const
+{
+   if (!A->is_integer_cell())
+      {
+        this->bif_reciprocal(Z);   // Z = ÷B
+        A->bif_multiply(Z, Z);     // Z = A × Z = A × ÷B
+        return;
+      }
+
+   // both cells are integers.
+   //
+const int64_t a = A->get_int_value();
+const int64_t b =    get_int_value();
+
+   if (b == 0)
+      {
+        if (a != 0)   DOMAIN_ERROR;
+        new (Z) IntCell(1);   // 0 ÷ 0 is defined as 1
+        return;
+      }
+
+const double i_quot = a / b;
+const double r_quot = a / (double)b;
+
+   if (r_quot > LARGE_INT || r_quot < SMALL_INT)   new (Z) FloatCell(r_quot);
+   else if (a != i_quot * b)                       new (Z) FloatCell(r_quot);
+   else                                            new (Z) IntCell(i_quot);
+}
+//-----------------------------------------------------------------------------
+void
+IntCell::bif_multiply(Cell * Z, const Cell * A) const
+{
+   if (!A->is_integer_cell())
+      {
+        A->bif_multiply(Z, this);
+        return;
+      }
+
+   // both cells are integers.
+   //
+const int64_t a = A->get_int_value();
+const int64_t b =    get_int_value();
+const double prod = a * (double)b;
+
+   if (prod > LARGE_INT || prod < SMALL_INT)   new (Z) FloatCell(prod);
+   else                                        new (Z) IntCell(a * b);
+}
+//-----------------------------------------------------------------------------
+void
+IntCell::bif_power(Cell * Z, const Cell * A) const
+{
+const int64_t b = get_int_value();
+   
+   if (!A->is_integer_cell())
+      {
+        FloatCell B(b);
+        B.bif_power(Z, A);
+        return;
+      }
+
+const int64_t a = A->get_int_value();
+
+   if (a == 0)   { new (Z) IntCell(0);   return; }   // 0^N = 0
+   if (a == 1)   { new (Z) IntCell(1);   return; }   // 1^N = 1
+   if (b == 0)   { new (Z) IntCell(1);   return; }   // N^0 = 1
+   if (b == 1)   { new (Z) IntCell(a);   return; }   // N^1 = N
+
+const double power = pow(a, b);
+   if (power > LARGE_INT || b < 0)
+      {
+        new (Z) FloatCell(power);
+        return;
+      }
+
+   // power is an int small enough for int_64
+   // a^b = a^(bn + bn-1 + ... + b0)         with   bj ∈ {0, 2^n}
+   //       a^(bn) × a^(bn-1) × ... × a^b0)  with a^bj ∈ {1, a(^2^n) = a^}
+
+int64_t a_2_n = a;  // a^(2^n) = a^(2^(n-1)) × a^(2^(n-1))
+int64_t ipow = 1;
+   for (int b1 = b; b1; b1 >>= 1)
+      {
+        if (b1 & 1)   ipow *= a_2_n;
+        a_2_n *= a_2_n;
+      }
+
+   new (Z) IntCell(ipow);
+}
+//-----------------------------------------------------------------------------
 void IntCell::bif_residue(Cell * Z, const Cell * A) const
 {
 const APL_Float qct = Workspace::get_CT();
@@ -292,7 +436,7 @@ const APL_Float qct = Workspace::get_CT();
         Z->bif_floor(Z);             // Z = A/B rounded down.
         new (Z) ComplexCell(b - a*Z->get_complex_value());
       }
-   else
+   else   // residue not defined for complex.
       {
         DOMAIN_ERROR;
       }
@@ -312,9 +456,9 @@ void IntCell::bif_maximum(Cell * Z, const Cell * A) const
          if (a >= APL_Float(value.ival))   new (Z) FloatCell(a);
          else                              new (Z) IntCell(value.ival);
       }
-   else   // maximum not defined for complex.
+   else
       {
-        DOMAIN_ERROR;
+        A->bif_maximum(Z, this);
       }
 }
 //-----------------------------------------------------------------------------
@@ -332,58 +476,9 @@ void IntCell::bif_minimum(Cell * Z, const Cell * A) const
          if (a <= APL_Float(value.ival))   new (Z) FloatCell(a);
          else                              new (Z) IntCell(value.ival);
       }
-   else   // minimum not defined for complex.
+   else
       {
-        DOMAIN_ERROR;
-      }
-}
-//-----------------------------------------------------------------------------
-void IntCell::bif_less_than(Cell * Z, const Cell * A) const
-{
-   if (A->is_integer_cell())
-      {
-        new (Z) IntCell(A->get_int_value() < value.ival ? 1 : 0);
-      }
-   else if (A->is_numeric())   // promote the request to float or complex
-      {
-        A->bif_greater_than(Z, this);
-      }
-   else   // char cell
-      {
-        DOMAIN_ERROR;
-      }
-}
-//-----------------------------------------------------------------------------
-void IntCell::bif_greater_than(Cell * Z, const Cell * A) const
-{
-   if (A->is_integer_cell())
-      {
-        new (Z) IntCell(A->get_int_value() > value.ival ? 1 : 0);
-      }
-   else if (A->is_numeric())   // promote the request to float or complex
-      {
-        A->bif_less_than(Z, this);
-      }
-   else   // char cell
-      {
-        DOMAIN_ERROR;
-      }
-}
-//-----------------------------------------------------------------------------
-void
-IntCell::bif_equal(Cell * Z, const Cell * A) const
-{
-   if (A->is_integer_cell())
-      {
-        new (Z) IntCell(A->get_int_value() == value.ival ? 1 : 0);
-      }
-   else if (A->is_numeric())   // promote the request to float or complex
-      {
-        A->bif_equal(Z, this);
-      }
-   else   // char cell
-      {
-        new (Z) IntCell(0);
+        A->bif_minimum(Z, this);
       }
 }
 //-----------------------------------------------------------------------------
