@@ -60,6 +60,8 @@ Nabla::throw_edit_error(const char * loc)
         << "      " << UCS_string(first_command.size() - 1, UNI_ASCII_SPACE)
         << "^" << endl;
 
+   Workspace::more_error() = UCS_string(loc);
+
    throw_define_error(fun_name, first_command, loc);
 }
 //-----------------------------------------------------------------------------
@@ -70,7 +72,7 @@ Nabla::edit()
       {
         CERR << "bad editor command '" << first_command
              << "' : problem '" << loc << "'" << endl;
-       NABLA_ERROR;
+        throw_edit_error(loc);
       }
 
    // editor loop
@@ -103,7 +105,8 @@ UCS_string fun_text;
       }
 
 int error_line = 0;
-UserFunction * ufun = UserFunction::fix(fun_text, error_line, false, LOC);
+UserFunction * ufun = UserFunction::fix(fun_text, error_line, false,
+                                        LOC, "∇ editor");
    if (ufun == 0)   NABLA_ERROR;
 
    if (locked)
@@ -256,8 +259,8 @@ command_loop:
    // [→   abandon
    switch (c.get())
       {
-        case UNI_QUAD_QUAD:
-        case UNI_QUAD_QUAD1:    ecmd = ECMD_SHOW;     c.next();   break;
+        case UNI_Quad_Quad:
+        case UNI_Quad_Quad1:    ecmd = ECMD_SHOW;     c.next();   break;
         case UNI_ASCII_R_BRACK: ecmd = ECMD_EDIT;                 break;
         case UNI_DELTA:         ecmd = ECMD_DELETE;   c.next();   break;
         case UNI_RIGHT_ARROW:   ecmd = ECMD_ESCAPE;   c.next();   break;
@@ -486,7 +489,8 @@ Nabla::open_new_function()
       CERR << "creating new function '" << fun_name << "'" << endl;
 
 int error_line = 0;
-UserFunction * ufun = UserFunction::fix(fun_name, error_line, true, LOC);
+UserFunction * ufun = UserFunction::fix(fun_name, error_line, true, LOC,
+                                        "∇ editor");
 
    if (ufun == 0)   return "Bad function header at " LOC;
    fun_name = ufun->get_name();
@@ -504,7 +508,7 @@ Nabla::execute_oper()
 const bool have_from = edit_from.ln_major != -1;
 const bool have_to = edit_to.ln_major != -1;
 
-   if (!have_from)   edit_from = lines[0].label;
+   if (!have_from && ecmd != ECMD_DELETE)   edit_from = lines[0].label;
    if (!have_to)     edit_to = lines[lines.size() - 1].label;
    if (edit_to.ln_major > lines.size() - 1)
       { edit_to.ln_major = lines.size() - 1;   edit_to.ln_minor.clear(); };
@@ -621,6 +625,7 @@ Nabla::execute_escape()
 {
    lines.clear();
    open_function();
+   return 0;
 }
 //-----------------------------------------------------------------------------
 int
