@@ -104,29 +104,26 @@ ostream & get_CERR()
 
 Output::ColorMode Output::color_mode = COLM_UNDEF;
 
+/// CSI sequence for ANSI/VT100 escape sequences (ESC [)
+#define CSI "\x1b["
+
 /// VT100 escape sequence to change to cin color
-char Output::color_CIN[21] =
- { 27, 91, '0', ';', '3', '0', ';', '4', '7', 'm', 0 };
+char Output::color_CIN[100] = CSI "0;30;47m";
 
 /// VT100 escape sequence to change to cout color
-char Output::color_COUT[21] =
-   { 27, 91, '0', ';', '3', '0', ';', '4', '8', 'm', 0 };
+char Output::color_COUT[100] = CSI "0;30;48;2;255;255;255m";
 
 /// VT100 escape sequence to change to cerr color
-char Output::color_CERR[21] =
-   { 27, 91, '0', ';', '3', '5', ';', '4', '8', 'm', 0 };
+char Output::color_CERR[100] = CSI "0;35;48;2;255;255;255m";
 
 /// VT100 escape sequence to change to cerr color
-char Output::color_UERR[21] =
-   { 27, 91, '0', ';', '3', '5', ';', '4', '8', 'm', 0 };
+char Output::color_UERR[100] = CSI "0;35;48;2;255;255;255m";
 
 /// VT100 escape sequence to reset colors to their default
-char Output::color_RESET[21] =
-  { 27, 91, '0', ';', '3', '8', ';', '4', '8', 'm', 0 };
+char Output::color_RESET[100] = CSI "0;30;48;2;255;255;255m";
 
 /// VT100 escape sequence to clear to end of line
-char Output::clear_EOL[21] =
-   { 27, 91, 'K', 0 };
+char Output::clear_EOL[100] = CSI "K";
 
 //-----------------------------------------------------------------------------
 int
