@@ -239,7 +239,7 @@ Prefix::unmark_all_values() const
         if (tok.get_ValueType() != TV_VAL)      continue;
 
         Value_P value = tok.get_apl_val();
-        if (!!value)   value->clear_marked();
+        if (!!value)   value->unmark();
       }
 }
 //-----------------------------------------------------------------------------
@@ -283,10 +283,8 @@ grow:
      Log(LOG_prefix_parser)
         {
           CERR << "    [si=" << si.get_level() << " PC=" << (PC - 1)
-               << "] Read token[" << size();
-          if (get_assign_state() == ASS_arrow_seen)      CERR << " ← ";
-          else if (get_assign_state() == ASS_var_seen)   CERR << " V← ";
-          CERR << "] " << tl.tok << " "
+               << "] Read token[" << size()
+               << "] (←" << get_assign_state() << "←) " << tl.tok << " "
                << Token::class_name(tl.tok.get_Class()) << endl;
         }
 
@@ -303,10 +301,6 @@ grow:
                //
                sym->resolve(tl.tok, true);
              }
-          else if (tl.tok.get_tag() == TOK_LSYMB)
-             {
-               sym->resolve(tl.tok, true);
-             }
           else
              {
                const bool left_sym = get_assign_state() == ASS_arrow_seen;
@@ -316,7 +310,7 @@ grow:
 
           if (tl.tok.get_tag() == TOK_SI_PUSHED)
             {
-              // Quad_QUAD::resolve() calls ⍎ which returns TOK_SI_PUSHED.
+              // Quad_Quad::resolve() calls ⍎ which returns TOK_SI_PUSHED.
               //
               push(tl);
               return Token(TOK_SI_PUSHED);
@@ -426,12 +420,12 @@ found_prefix:
      switch (at0().get_Class())
         {
           case TC_VALUE:
-               more = (best->prio < BS_VAL_VAL) &&
+               more = ((best->prio < BS_VAL_VAL) &&
                         (  next == TC_VALUE            // A B
                         || next == TC_R_PARENT         // ) B
-                        ) ||
+                        )) ||
 
-                      best->prio < BS_OP_RO && next == TC_OPER2;
+                      (best->prio < BS_OP_RO && next == TC_OPER2);
 
                break;
 
@@ -573,6 +567,7 @@ Prefix::reduce_N___()
 
 Token result = si.eval_(at0());
    move_1(at0(), result, LOC);
+   at0().SET_temp();
    set_action(at0());
 }
 //-----------------------------------------------------------------------------
@@ -582,6 +577,7 @@ Prefix::reduce_MISC_F_B_()
    Assert1(prefix_len == 2);
 
    pop_args_push_result(si.eval_B(at0(), at1()));
+   at0().SET_temp();
    set_action(at0());
 }
 //-----------------------------------------------------------------------------
@@ -590,7 +586,11 @@ Prefix::reduce_MISC_F_C_B()
 {
    Assert1(prefix_len == 3);
 
+   if (at1().get_ValueType() != TV_VAL)   SYNTAX_ERROR;
+   if (!at1().get_apl_val())              SYNTAX_ERROR;
+
    pop_args_push_result(si.eval_XB(at0(), at1(), at2()));
+   at0().SET_temp();
    set_action(at0());
 }
 //-----------------------------------------------------------------------------
@@ -600,6 +600,7 @@ Prefix::reduce_A_F_B_()
    Assert1(prefix_len == 3);
 
    pop_args_push_result(si.eval_AB(at0(), at1(), at2()));
+   at0().SET_temp();
    set_action(at0());
 }
 //-----------------------------------------------------------------------------
@@ -616,7 +617,11 @@ Prefix::reduce_A_F_C_B()
 {
    Assert1(prefix_len == 4);
 
+   if (at2().get_ValueType() != TV_VAL)   SYNTAX_ERROR;
+   if (!at2().get_apl_val())              SYNTAX_ERROR;
+
    pop_args_push_result(si.eval_AXB(at0(), at1(), at2(), at3()));
+   at0().SET_temp();
    set_action(at0());
 }
 //-----------------------------------------------------------------------------
@@ -627,6 +632,12 @@ Prefix::reduce_A_M_C_B()
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_N_M__()
+{
+   reduce_F_M__();
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_F_M__()
 {
 DerivedFunction * derived =
@@ -638,6 +649,18 @@ DerivedFunction * derived =
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_F_C_M_()
+{
+DerivedFunction * derived =
+   Workspace::SI_top()->fun_oper_cache.get(LOC);
+   new (derived) DerivedFunction(at0(), at2().get_function(),
+                                 at1().get_axes(), LOC);
+
+   pop_args_push_result(Token(TOK_FUN2, derived));
+   action = RA_CONTINUE;
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_B_D_B_()
 {
    reduce_F_D_G_();   // same as F2
@@ -650,12 +673,24 @@ Prefix::reduce_B_D_G_()
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_B_D_N_()
+{
+   reduce_F_D_G_();   // same as F2
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_F_D_B_()
 {
    reduce_F_D_G_();   // same as G2
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_N_D_B_()
+{
+   reduce_F_D_G_();   // same as G2
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_F_D_G_()
 {
 DerivedFunction * derived =
@@ -667,6 +702,24 @@ DerivedFunction * derived =
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_F_D_N_()
+{
+   reduce_F_D_G_();   // same as G2
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_N_D_G_()
+{
+   reduce_F_D_G_();   // same as G2
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_N_D_N_()
+{
+   reduce_F_D_G_();   // same as G2
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_A_C__()
 {
 Value_P A = at0().get_apl_val();
@@ -675,6 +728,7 @@ Value_P Z = A->index(at1());
 Token result = Token(TOK_APL_VALUE1, Z);
    pop_args_push_result(result);
 
+   at0().SET_temp();
    set_action(result);
 }
 //-----------------------------------------------------------------------------
@@ -784,8 +838,7 @@ Prefix::reduce_RBRA___()
    // A[IDX}←B resolves IDX properly. assign_state is restored when the
    // index is complete.
    //
-const int left = get_assign_state() == ASS_arrow_seen;
-   new (&at0()) Token(TOK_PINDEX, *new IndexExpr(left, LOC));
+   new (&at0()) Token(TOK_PINDEX, *new IndexExpr(get_assign_state(), LOC));
    set_assign_state(ASS_none);
    action = RA_CONTINUE;
 }
@@ -802,7 +855,7 @@ const bool last_index = (at0().get_tag() == TOK_L_BRACK);
 
    if (idx.value_count() == 0 && last_index)   // special case: [ ]
       {
-        assign_state = idx.get_left() ? ASS_arrow_seen : ASS_none;
+        assign_state = idx.get_assign_state();
         Token result = Token(TOK_INDEX, idx);
         pop_args_push_result(result);
         action = RA_CONTINUE;
@@ -816,7 +869,7 @@ Token result = at1();
 
    if (last_index)   // [ seen
       {
-        assign_state = idx.get_left() ? ASS_arrow_seen : ASS_none;
+        assign_state = idx.get_assign_state();
 
         if (idx.is_axis()) move_2(result, Token(TOK_AXES, idx.values[0]), LOC);
         else               move_2(result, Token(TOK_INDEX, idx), LOC);
@@ -845,7 +898,7 @@ const bool last_index = (at0().get_tag() == TOK_L_BRACK);   // ; vs. [
    if (last_index)   // [ seen
       {
         IndexExpr & idx = I.get_index_val();
-        assign_state = idx.get_left() ? ASS_arrow_seen : ASS_none;
+        assign_state = idx.get_assign_state();
 
         if (idx.is_axis())
            {
@@ -877,6 +930,7 @@ Token result;
    Value::glue(result, at0(), at1(), LOC);
    pop_args_push_result(result);
 
+   at0().SET_temp();
    set_action(result);
 }
 //-----------------------------------------------------------------------------
@@ -1156,7 +1210,7 @@ Prefix::reduce_RETC___()
 
    // not reached
    //
-   Q(at0().get_tag())   FIXME;
+   Q1(at0().get_tag())   FIXME;
 }
 //-----------------------------------------------------------------------------
 // Note: reduce_RETC_B___ happens only for context ⍎,
