@@ -48,7 +48,7 @@ Quad_SVQ Quad_SVQ::fun;
 Quad_SVR Quad_SVR::fun;
 Quad_SVS Quad_SVS::fun;
 
-APL_time Quad_SVE::timer_end = 0;
+APL_time_us Quad_SVE::timer_end = 0;
 
 //=============================================================================
 /**
@@ -115,8 +115,6 @@ const AP_num par_ID = ProcessorID::get_parent_ID();
 
    CERR << "No binary found for AP " << proc << " (interpreter path = "
         << LibPaths::get_APL_bin_path() << ")" << endl;
-
-   return;
 }
 //=============================================================================
 Token
@@ -211,7 +209,7 @@ Value_P Z(var_count > 1 ? new Value(shZ, LOC) : new Value(4, LOC));
 }
 //=============================================================================
 Quad_SVE::Quad_SVE()
-   : NL_SystemVariable(ID_QUAD_SVE)
+   : NL_SystemVariable(ID_Quad_SVE)
 {
    Symbol::assign(Value::Zero_P, LOC);
 }
@@ -224,7 +222,7 @@ Quad_SVE::assign(Value_P value, const char * loc)
    //
    if (!value->is_skalar())   RANK_ERROR;
 
-const APL_time duration = 1000000 * value->get_ravel(0).get_real_value();
+const APL_time_us duration = 1000000 * value->get_ravel(0).get_real_value();
    if (duration < 0)   DOMAIN_ERROR;
 
    if (duration == 0.0)
@@ -244,8 +242,8 @@ const APL_time duration = 1000000 * value->get_ravel(0).get_real_value();
 Value_P
 Quad_SVE::get_apl_value() const
 {
-const APL_time current_time = now();
-const APL_time wait = timer_end - current_time;
+const APL_time_us current_time = now();
+const APL_time_us wait = timer_end - current_time;
 
    // if timer has expired, return 0
    //
@@ -378,7 +376,7 @@ Value_P Z(var_count > 1 ? new Value(var_count, LOC) : new Value(LOC));
                   continue;   // next z
 
              default: // function etc.
-                  Q(sym->get_nc())   DOMAIN_ERROR;
+                  Q1(sym->get_nc())   DOMAIN_ERROR;
            }
 
         SV_Coupling coupling = NO_COUPLING;
@@ -404,7 +402,7 @@ Quad_SVO::share_one_variable(AP_num to_ap, const uint32_t * vname,
 AP_num3 to_proc(to_ap, ProcessorID::get_own_ID(), ProcessorID::get_parent_ID());
 bool auto_started = false;
 
-   if (to_ap < AP_FIRST_USER)   // to_proc is an AP
+   if (to_ap && to_ap < AP_FIRST_USER)   // to_proc is an AP
       {
         if (!Svar_DB::is_registered(to_proc))
            {
