@@ -46,15 +46,49 @@ const UCS_string symbol_name(*B.get());
 
 Value_P Z;
 
-   if (mode == 1)        Z = tf1(symbol_name);
-   else if (mode == 2)   return tf2(symbol_name);
-   else if (mode == 3)   Z = tf3(symbol_name);
-   else                  DOMAIN_ERROR;
+   if (mode == 1)
+      {
+        const bool inverse = is_inverse(symbol_name);
+        if (inverse)   Z = tf1_inv(symbol_name);
+        else           Z = tf1(symbol_name);
+      }
+   else if (mode == 2)
+      {
+        const bool inverse = is_inverse(symbol_name);
+        if (!inverse)   return tf2(symbol_name);
+        Z = Value_P(new Value(tf2_inv(symbol_name), LOC));
+      }
+   else if (mode == 3)
+      {
+        Z = tf3(symbol_name);
+      }
+   else
+      {
+        DOMAIN_ERROR;
+      }
 
    Z->check_value(LOC);
    return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
+bool
+Quad_TF::is_inverse(const UCS_string & maybe_name)
+{
+   // a forward ⎕TF contains only a function or variable name.
+   // an inverse ⎕TF contains at least one non-symbol character
+   //
+   loop(n, maybe_name.size())
+      {
+        if (n == 0 && Avec::is_quad(maybe_name[n]))   continue;
+        if (Avec::is_symbol_char(maybe_name[n]))      continue;
+
+        return true;   // inverse ⎕TF
+      }
+
+   // all chars in val were symbol characters. Therefore it is a forward ⎕TF
+   return false;
+}
+//-----------------------------------------------------------------------------
 Value_P
 Quad_TF::tf1(const UCS_string & name)
 {
@@ -86,13 +120,6 @@ Symbol * symbol = obj->get_symbol();
 Value_P
 Quad_TF::tf1(const UCS_string & var_name, Value_P val)
 {
-   // check for inverse 1 ⎕TF and return it if applicable.
-   //
-   {
-     Value_P Z = tf1_inv(val);
-     if (!!Z)   return Z;
-   }
-
 const bool is_char_array = val->get_ravel(0).is_character_cell();
 UCS_string ucs(is_char_array ? UNI_ASCII_C : UNI_ASCII_N);
 
@@ -137,7 +164,7 @@ const ShapeItem ec = val->element_count();
                 }
              else // if (cell.is_near_real(qct))
                 {
-                  PrintContext pctx(PR_APL_MIN, MAX_QUAD_PP, 0.0, MAX_QUAD_PW);
+                  PrintContext pctx(PR_APL_MIN, MAX_Quad_PP, 0.0, MAX_Quad_PW);
                   const APL_Float value = cell.get_real_value();
                   bool scaled = true;
                   UCS_string ucs1(value, scaled, pctx);
@@ -177,12 +204,8 @@ UCS_string ucs;
 }
 //-----------------------------------------------------------------------------
 Value_P
-Quad_TF::tf1_inv(Value_P val)
+Quad_TF::tf1_inv(const UCS_string & ravel)
 {
-   if (!val->is_char_vector())   return Value_P();
-
-UCS_string ravel(*val.get());
-
 const size_t len = ravel.size();
    if (len < 2)   return Value_P();
 
@@ -351,22 +374,6 @@ Symbol * symbol = obj->get_symbol();
 Token
 Quad_TF::tf2_var(const UCS_string & var_name, Value_P value)
 {
-   // try inverse 2 ⎕TF and return it if successful
-   //
-   if (value->is_apl_char_vector())
-      {
-        UCS_string data(*value.get());
-        UCS_string new_var_or_fun;
-
-        tf2_parse(data, new_var_or_fun);
-        if (new_var_or_fun.size())   // if data is an inverse TF2
-           {
-             Value_P Z(new Value(new_var_or_fun, LOC));
-             Z->check_value(LOC);
-             return Token(TOK_APL_VALUE1, Z);
-          }
-      }
-
 UCS_string ucs(var_name);
    ucs.append(UNI_LEFT_ARROW);
 
@@ -381,49 +388,61 @@ Value_P Z(new Value(ucs, LOC));
    return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
-void
-Quad_TF::tf2_parse(const UCS_string & ucs, UCS_string & new_var_or_fun)
+UCS_string
+Quad_TF::tf2_inv(const UCS_string & ravel)
 {
 Token_string tos;
-Parser parser(PM_EXECUTE, LOC);
 
-   try          { parser.parse(ucs, tos); }
-   catch(...)   { return; }
+   try
+      {
+        UCS_string ucs1 = no_UCS(ravel);
+        const Parser parser(PM_EXECUTE, LOC);
+        parser.parse(ucs1, tos);
+      }
+   catch(...)
+      {
+        return UCS_string();
+      }
 
-   if (tos.size() < 2)   return;   // too short for an inverse 2⎕TF
+   // simplify tos as much as possible. We replace A⍴B by reshaped B
+   // and glue values together.
+   //
+   tf2_simplify(tos);
 
-   // we expect either VAR←VALUE ... or ⎕FX fun-text. Try VAR←VALUE.
+
+   if (tos.size() < 2)   return UCS_string();   // too short for an inverse 2⎕TF
+
+   // it could happen that some system variable ⎕XX which is not known by
+   // GNU APL is read. This case is parsed as ⎕ XX ← ... 
+   // Issue a warning in that case
+   //
+   if (tos[0].get_tag() == TOK_Quad_Quad &&
+       tos[1].get_Class() == TC_SYMBOL)
+      {
+        UCS_string new_var_or_fun = tos[1].get_sym_ptr()->get_name();
+        CERR << "*** Unknown system variable ⎕" << new_var_or_fun
+             << " in 2⎕TF / )IN (assignment ignored)" << endl;
+        return new_var_or_fun;
+      }
+
+   // we expect either VAR ← VALUE ... or ⎕FX fun-text. Try VAR ← VALUE.
    //
    if (tos[0].get_Class() == TC_SYMBOL && tos[1].get_tag() == TOK_ASSIGN)
       {
-        // simplify tos as much as possible. We replace A⍴B by reshaped B
-        // and glue values together.
+        // at this point, we expect SYM ← VALUE.
         //
-        tf2_remove_UCS(tos);
-
-        for (bool progress = true; progress;)
-            {
-              progress = false;
-              tf2_remove_RHO(tos, progress);
-              tf2_remove_parentheses(tos, progress);
-              tf2_glue(tos, progress);
-            }
-
-        // at this point, we expect SYM←VALUE.
-        //
-        if (tos.size() != 3)                  return;
-        if (tos[2].get_Class() != TC_VALUE)   return;
+        if (tos.size() != 3)                  return UCS_string();
+        if (tos[2].get_Class() != TC_VALUE)   return UCS_string();
 
          tos[0].get_sym_ptr()->assign(tos[2].get_apl_val(), LOC);
-         new_var_or_fun = tos[0].get_sym_ptr()->get_name();   // valid 2⎕TF
-         return;
+         return tos[0].get_sym_ptr()->get_name();   // valid 2⎕TF
       }
 
    // Try ⎕FX fun-text
    //
-   if (tos.size() != 2)                   return;
-   if (tos[0].get_tag() != TOK_QUAD_FX)   return;
-   if (tos[1].get_Class() != TC_VALUE)    return;
+   if (tos.size() != 2)                   return UCS_string();
+   if (tos[0].get_tag() != TOK_Quad_FX)   return UCS_string();
+   if (tos[1].get_Class() != TC_VALUE)    return UCS_string();
 
    {
      Value_P function_text =  tos[1].get_apl_val();
@@ -432,9 +451,11 @@ Parser parser(PM_EXECUTE, LOC);
    if (tok.get_Class() == TC_VALUE)   // ⎕FX successful
       {
         Value_P val = tok.get_apl_val();
-        new_var_or_fun = UCS_string(*val.get());
+        return UCS_string(*val.get());
       }
    }
+
+   return UCS_string();
 }
 //-----------------------------------------------------------------------------
 void
@@ -495,22 +516,24 @@ const ShapeItem ec = value->element_count();
       {
         tf2_shape(ucs, value->get_shape());
 
-        // if all elements are in ⎕AV then print 'elements' otherwise use ⎕UCS.
+        // check if ⎕UCS is needed. 
+        //
+        // ⎕UCS is needed if the string contains a character that is:
+        //
+        // 1. not in our ⎕AV, or
+        // 2. not in IBM's ⎕AV
+        //
         bool use_UCS = false;
         loop(e, ec)
            {
              const Unicode uni = value->get_ravel(e).get_char_value();
-             if (Avec::find_char(uni) == Invalid_CHT)   // uni not in ⎕AV
-                {
-                  use_UCS = true;
-                  break;
-                }
+             if (Avec::need_UCS(uni))   { use_UCS = true;   break; }
            }
 
         if (use_UCS)
            {
              if (level)   ucs.append(UNI_ASCII_L_PARENT);
-             ucs.append(UNI_QUAD_QUAD);
+             ucs.append(UNI_Quad_Quad);
              ucs.append(UNI_ASCII_U);
              ucs.append(UNI_ASCII_C);
              ucs.append(UNI_ASCII_S);
@@ -559,7 +582,7 @@ const Depth depth = value->compute_depth();
            }
         else if (cell.is_complex_cell())
            {
-             PrintContext pctx(PR_APL_MIN, MAX_QUAD_PP, 0.0, MAX_QUAD_PW);
+             PrintContext pctx(PR_APL_MIN, MAX_Quad_PP, 0.0, MAX_Quad_PW);
              bool scaled = true;
              UCS_string ucs1(cell.get_real_value(), scaled, pctx);
              UCS_string ucs2(cell.get_imag_value(), scaled, pctx);
@@ -569,7 +592,7 @@ const Depth depth = value->compute_depth();
            }
         else if (cell.is_float_cell())
            {
-             PrintContext pctx(PR_APL_MIN, MAX_QUAD_PP, 0.0, MAX_QUAD_PW);
+             PrintContext pctx(PR_APL_MIN, MAX_Quad_PP, 0.0, MAX_Quad_PW);
              bool scaled = true;
              UCS_string ucs1(cell.get_real_value(), scaled, pctx);
              ucs.append(ucs1);
@@ -589,6 +612,23 @@ const Depth depth = value->compute_depth();
 }
 //-----------------------------------------------------------------------------
 int
+Quad_TF::tf2_simplify(Token_string & tos)
+{
+int errors = 0;
+
+   for (bool progress = true; progress && !errors;)
+       {
+         progress = false;
+         errors += tf2_remove_UCS(tos);
+         errors += tf2_remove_RHO(tos, progress);
+         errors += tf2_remove_parentheses(tos, progress);
+         errors += tf2_glue(tos, progress);
+       }
+
+   return errors;
+}
+//-----------------------------------------------------------------------------
+int
 Quad_TF::tf2_remove_UCS(Token_string & tos)
 {
 int d = 0;
@@ -596,7 +636,7 @@ bool ucs_active = false;
 
    loop(s, tos.size())
       {
-        if (s < (tos.size() - 1) && tos[s].get_tag() == TOK_QUAD_UCS)
+        if (s < (tos.size() - 1) && tos[s].get_tag() == TOK_Quad_UCS)
            {
               ucs_active = true;
               continue;
@@ -648,7 +688,7 @@ int d = 0;
                 }
              else
                 {
-                  move_2(tos[d], Bif_F12_RHO::fun.do_reshape(sh, bval), LOC);
+                  move_2(tos[d], Bif_F12_RHO::do_reshape(sh, bval), LOC);
                   progress = true;
                   ++d;
                   s += 2;
@@ -737,24 +777,84 @@ const UCS_string text = fun.canonical(false);
 vector<UCS_string> lines;
    text.to_vector(lines);
 
-   ucs.append(UNI_QUAD_QUAD);
-   ucs.append(UNI_ASCII_F);
-   ucs.append(UNI_ASCII_X);
+   ucs.append_utf8("⎕FX");
 
    loop(l, lines.size())
       {
         ucs.append(UNI_ASCII_SPACE);
-        ucs.append(UNI_SINGLE_QUOTE);
+        tf2_char_vec(ucs, lines[l]);
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Quad_TF::tf2_char_vec(UCS_string & ucs, const UCS_string & vec)
+{
+   if (vec.size() == 0)   return;
 
-        loop(c, lines[l].size())
+bool in_UCS = false;
+   ucs.append_utf8("'");
+
+   loop(v, vec.size())
+       {
+         const Unicode uni = vec[v];
+         const bool need_UCS = Avec::need_UCS(uni);
+         if (in_UCS != need_UCS)   // mode changed
             {
-              const Unicode uni = lines[l][c];
-              ucs.append(uni);
-              if (uni == UNI_SINGLE_QUOTE)   ucs.append(UNI_SINGLE_QUOTE);
+              if (in_UCS)   ucs.append_utf8("),'");      // UCS() → 'xxx'
+              else          ucs.append_utf8("',(⎕UCS");   // 'xxx' → UCS()
+              in_UCS = need_UCS;
             }
-        ucs.append(UNI_SINGLE_QUOTE);
+
+         if (in_UCS)
+            {
+              ucs.append_utf8(" ");
+              ucs.append_number(uni);
+            }
+         else
+            {
+              ucs.append(uni);
+              if (uni == UNI_SINGLE_QUOTE)   ucs.append(uni);
+            }
+       }
+
+   if (in_UCS)   ucs.append_utf8("),''");
+   else          ucs.append_utf8("'");
+}
+//-----------------------------------------------------------------------------
+UCS_string
+Quad_TF::no_UCS(const UCS_string & ucs)
+{
+UCS_string ret;
+
+   loop(u, ucs.size())
+      {
+        if ( u < ucs.size() - 6 &&
+             ucs[u]     == '\'' &&
+             ucs[u + 1] == ','  &&
+             ucs[u + 2] == '('  &&
+             ucs[u + 3] == UNI_Quad_Quad &&
+             ucs[u + 4] == 'U'  &&
+             ucs[u + 5] == 'C'  &&
+             ucs[u + 6] == 'S')
+           {
+             u += 7;   // skip '(⎕UCS
+
+             for (;;)
+                 {
+                  while (u < ucs.size() && ucs[u] == ' ')   ++u;
+                  if (ucs[u] == ')')   { u += 2;   break; }
+
+                  int num = 0;
+                  while (u < ucs.size() && ucs[u] >= '0' && ucs[u] <= '9')
+                     { num *= 10;   num += ucs[u++] - '0'; }
+
+                  ret.append((Unicode)num);
+                 }
+           }
+        else ret.append(ucs[u]);
       }
 
+   return ret;
 }
 //-----------------------------------------------------------------------------
 Value_P
