@@ -20,6 +20,7 @@
 
 #include <iomanip>
 
+#include "Command.hh"
 #include "Executable.hh"
 #include "IndexExpr.hh"
 #include "Input.hh"
@@ -38,6 +39,7 @@ StateIndicator::StateIndicator(const Executable * exec, StateIndicator * _par)
      level(_par ? 1 + _par->get_level() : 0),
      error(E_NO_ERROR, LOC),
      current_stack(*this, exec->get_body()),
+     eval_arg_F(0),
      parent(_par)
 {
 }
@@ -100,6 +102,14 @@ StateIndicator::set_args(Token * A, Token & F, Token * X, Token * B)
 
    if (X)
       {
+        // X can be a single value (which is OK) or an IndexExpr (which is not)
+        //
+        if (X->get_ValueType() != TV_VAL)
+           {
+             IndexExpr & idx = X->get_index_val();
+             idx.extract_all();   // clear all values in X
+             SYNTAX_ERROR;
+           }
         eval_arg_X = X->get_axes();
       }
 
@@ -487,9 +497,9 @@ Token result = current_stack.reduce_statements();
 void
 StateIndicator::unmark_all_values() const
 {
-   if (!!eval_arg_A)   eval_arg_A->clear_marked();
-   if (!!eval_arg_X)   eval_arg_X->clear_marked();
-   if (!!eval_arg_B)   eval_arg_B->clear_marked();
+   if (!!eval_arg_A)   eval_arg_A->unmark();
+   if (!!eval_arg_X)   eval_arg_X->unmark();
+   if (!!eval_arg_B)   eval_arg_B->unmark();
 
    Assert(executable);
    executable->unmark_all_values();
@@ -517,7 +527,7 @@ char cc[100];
    count += executable->show_owners(cc, out, value);
 
    snprintf(cc, sizeof(cc), "    SI[%d] ", level);
-   current_stack.show_owners(cc, out, value);
+   count += current_stack.show_owners(cc, out, value);
 
    return count;
 }
@@ -892,24 +902,32 @@ StateIndicator::statement_result(Token & result)
 
    fun_oper_cache.reset();
 
-   if (get_executable()->get_parse_mode() == PM_EXECUTE)   return;
+// if (get_executable()->get_parse_mode() == PM_EXECUTE)   return;
 
    // if result is a value then print it, unless it is a committed value
    // (i.e. TOK_APL_VALUE2)
    //
-   if (result.get_ValueType() == TV_VAL)
-      {
-        const TokenTag tag = result.get_tag();
-        Value_P B(result.get_apl_val());
-        Assert(!!B);
+   if (result.get_ValueType() != TV_VAL)   return;
 
-        // print TOK_APL_VALUE and TOK_APL_VALUE1, but not TOK_APL_VALUE2
-        //
-        if (tag == TOK_APL_VALUE1 || tag == TOK_APL_VALUE3)
+const TokenTag tag = result.get_tag();
+Value_P B(result.get_apl_val());
+   Assert(!!B);
+
+   // print TOK_APL_VALUE and TOK_APL_VALUE1, but not TOK_APL_VALUE2
+   //
+   if (tag == TOK_APL_VALUE1 || tag == TOK_APL_VALUE3)
+      {
+        Quad_QUOTE::done(false, LOC);
+        const int boxing_format = Command::get_boxing_format();
+        if (boxing_format == -1)
            {
-             Quad_QUOTE::done(false, LOC);
              B->print(COUT);
            }
+        else
+           {
+             Value_P B1 = Quad_CR::do_CR(boxing_format, *B.get());
+             B1->print(COUT);
+           }
       }
 }
 //-----------------------------------------------------------------------------
