@@ -45,7 +45,8 @@ Symbol::Symbol(const UCS_string & ucs, Id id)
    : NamedObject(id),
      next(0),
      symbol(ucs),
-     erased(false)
+     erased(false),
+     monitor_callback(0)
 {
    push();
 }
@@ -129,6 +130,7 @@ ValueStackItem & vs = value_stack.back();
 
              vs.name_class = NC_VARIABLE;
              vs.apl_val = new_value;
+             if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
              return;
 
         case NC_VARIABLE:
@@ -141,19 +143,29 @@ ValueStackItem & vs = value_stack.back();
              Workspace::replace_arg(vs.apl_val, new_value);
 
              vs.apl_val = new_value;
+             if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
              return;
 
         case NC_SHARED_VAR:
              {
                CDR_string cdr;
                CDR::to_CDR(cdr, new_value);
+               if (cdr.size() > MAX_SVAR_SIZE)   LIMIT_ERROR_SVAR;
                string data((const char *)cdr.get_items(), cdr.size());
                const SV_key key = get_SV_key();
 
                // wait for shared variable to be ready
                //
                offered_SVAR * svar =  Svar_DB::find_var(get_SV_key());
-               Assert(svar);
+               if (svar == 0)
+                  {
+                    Log(LOG_shared_variables)
+                       CERR << " Svar_DB::find_var() failed." << endl;
+
+                    Workspace::more_error() =
+                       UCS_string("Svar_DB corrupt or erased?");
+                    VALUE_ERROR;
+                  }
 
                for (int w = 0; ; ++w)
                    {
@@ -218,6 +230,7 @@ ValueStackItem & vs = value_stack.back();
                     }
                }
              }
+             if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
              return;
       }
              
@@ -244,6 +257,7 @@ const ShapeItem max_idx = A->element_count();
               dest.release(LOC);   // free sub-values etc (if any)
               dest.init(src);
             }
+        if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
         return;
       }
 
@@ -268,10 +282,12 @@ const Cell * cB = &B->get_ravel(0);
 
          cB += incr_B;
       }
+
+   if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
 }
 //-----------------------------------------------------------------------------
 void
-Symbol::assign_indexed(const IndexExpr & IX, Value_P B)   // A[IX;...] ← B
+Symbol::assign_indexed(IndexExpr & IX, Value_P B)   // A[IX;...] ← B
 {
    if (IX.value_count() == 1 && !!IX.values[0])   // one-dimensional index
       {
@@ -346,6 +362,8 @@ const int incr_B = (ec_B == 1) ? 0 : 1;
         dest.init(*cB);
         cB += incr_B;
      }
+
+   if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -368,6 +386,7 @@ const ValueStackItem & vs = value_stack.back();
         if (erase)   ptr_clear(ret, LOC);
 
         value_stack.pop_back();
+        if (monitor_callback)   monitor_callback(*this, SEV_POPED);
         return ret;
       }
    else
@@ -380,6 +399,7 @@ const ValueStackItem & vs = value_stack.back();
              CERR << endl;
            }
         value_stack.pop_back();
+        if (monitor_callback)   monitor_callback(*this, SEV_POPED);
         return Value_P();
       }
 }
@@ -395,6 +415,7 @@ Symbol::push()
       }
 
    value_stack.push_back(ValueStackItem());
+   if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
 }
 //-----------------------------------------------------------------------------
 void
@@ -408,6 +429,7 @@ Symbol::push_label(Function_Line label)
       }
 
    value_stack.push_back(ValueStackItem(label));
+   if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
 }
 //-----------------------------------------------------------------------------
 void
@@ -425,6 +447,7 @@ ValueStackItem vs;
    else                           vs.name_class = NC_FUNCTION;
    vs.sym_val.function = function;
    value_stack.push_back(vs);
+   if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
 }
 //-----------------------------------------------------------------------------
 void
@@ -432,6 +455,7 @@ Symbol::push_value(Value_P value)
 {
 ValueStackItem vs;
    value_stack.push_back(vs);
+   if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
    assign(value, LOC);
 
    Log(LOG_SYMBOL_push_pop)
@@ -556,7 +580,7 @@ Symbol::get_attributes(int mode, Cell * dest) const
 {
 const ValueStackItem & vs = value_stack.back();
 bool is_var = false;
-int has_result = 0;
+bool has_result = false;
 int fun_valence = 0;
 int oper_valence = 0;
 double created = 0.0;
@@ -568,7 +592,7 @@ const int * exec_properties = exec_prop;
         case NC_LABEL:
         case NC_VARIABLE:
              is_var = 1;
-             has_result = 1;
+             has_result = true;
              break;
 
         case NC_FUNCTION:
@@ -589,7 +613,7 @@ const int * exec_properties = exec_prop;
    switch(mode)
       {
         case 1: // valences
-                new (dest + 0) IntCell(has_result);
+                new (dest + 0) IntCell(has_result ? 1 : 0);
                 new (dest + 1) IntCell(fun_valence);
                 new (dest + 2) IntCell(oper_valence);
                 break;
@@ -692,7 +716,16 @@ Symbol::resolve(Token & tok, bool left_sym)
                // wait for shared variable to be ready
                //
                offered_SVAR * svar =  Svar_DB::find_var(key);
-               Assert(svar);
+               if (svar == 0)
+                  {
+                    Log(LOG_shared_variables)
+                       CERR << " Svar_DB::find_var() failed." << endl;
+
+                    Workspace::more_error() =
+                       UCS_string("Svar_DB corrupt or erased?");
+                    VALUE_ERROR;
+                  }
+
 
                for (int w = 0; ; ++w)
                    {
@@ -761,7 +794,16 @@ Symbol::resolve(Token & tok, bool left_sym)
                // update shared var state (AFTER sending request to peer)
                {
                  offered_SVAR * svar =  Svar_DB::find_var(get_SV_key());
-                 Assert(svar);
+                 if (svar == 0)
+                    {
+                      Log(LOG_shared_variables)
+                         CERR << " Svar_DB::find_var() failed." << endl;
+
+                      Workspace::more_error() =
+                         UCS_string("Svar_DB corrupt or erased?");
+                      VALUE_ERROR;
+                    }
+
                  svar->set_state(true, LOC);
                }
                value->check_value(LOC);
@@ -843,26 +885,29 @@ ValueStackItem & vs = value_stack.back();
       }
    else if (vs.name_class == NC_FUNCTION || vs.name_class == NC_OPERATOR)
       {
-        const UserFunction * ufun = vs.sym_val.function->get_ufun1();
-        Assert(ufun);
-        const Executable * exec = ufun;
-        StateIndicator * oexec = Workspace::oldest_exec(exec);
-        if (oexec)
+        if (!vs.sym_val.function->is_native())
            {
-             // ufun is still used on the SI stack. We do not delete ufun,
-             // but merely remember it for deletion later on.
-             //
-             // CERR << "⎕EX function " << ufun->get_name()
-             //      << " is on SI !" << endl;
-             Workspace::add_expunged_function(ufun);
-           }
-        else
-           {
-             delete ufun;
+             const UserFunction * ufun = vs.sym_val.function->get_ufun1();
+             const Executable * exec = ufun;
+             StateIndicator * oexec = Workspace::oldest_exec(exec);
+             if (oexec)
+                {
+                  // ufun is still used on the SI stack. We do not delete ufun,
+                  // but merely remember it for deletion later on.
+                  //
+                  // CERR << "⎕EX function " << ufun->get_name()
+                  //      << " is on SI !" << endl;
+                  Workspace::add_expunged_function(ufun);
+                }
+             else
+                {
+                  delete ufun;
+                }
            }
       }
 
    vs.clear();
+   if (monitor_callback)   monitor_callback(*this, SEV_ERASED);
    return 1;
 }
 //-----------------------------------------------------------------------------
@@ -929,25 +974,21 @@ const SV_Coupling coupling = Svar_DB::retract_var(key);
    return coupling;
 }
 //-----------------------------------------------------------------------------
-void Symbol::set_nc(NameClass nc, Function * fun)
+void
+Symbol::set_nc(NameClass nc, Function * fun)
 {
 ValueStackItem & vs = value_stack.back();
 
-   if (vs.name_class == NC_UNUSED_USER_NAME)   // new function
-      {
-        vs.name_class = nc;
-        vs.sym_val.function = fun;
-        return;
-      }
+const bool can_set = (vs.name_class == NC_FUNCTION) ||
+                     (vs.name_class == NC_OPERATOR) ||
+                     (vs.name_class == NC_UNUSED_USER_NAME);
+             
+   Assert(nc == NC_FUNCTION || NC_OPERATOR || NC_UNUSED_USER_NAME);
 
-   if (vs.name_class == nc)   // existing function
-      {
-//      delete vs.sym_val.function;
-        vs.sym_val.function = fun;
-        return;
-      }
-
-   DEFN_ERROR;
+   if (!can_set)   DEFN_ERROR;
+   vs.sym_val.function = fun;
+   if (fun)   vs.name_class = nc;
+   else       vs.name_class = NC_UNUSED_USER_NAME;
 }
 //-----------------------------------------------------------------------------
 ostream &
@@ -1140,6 +1181,53 @@ const Cell * cV = &values->get_ravel(0);
 }
 //-----------------------------------------------------------------------------
 void
+Symbol::dump(ostream & out) const
+{
+const ValueStackItem & vs = (*this)[0];
+   if (vs.name_class == NC_VARIABLE)
+      {
+        vector<UCS_string> CR10;
+        const UCS_string pick;
+        Quad_CR::do_CR10_var(CR10, get_name(), pick, *vs.apl_val.get());
+
+        loop(l, CR10.size())
+           {
+             if (l)   out << "  ";
+             out << CR10[l] << endl;
+           }
+        out << endl;
+      }
+   else if (vs.name_class == NC_FUNCTION ||
+            vs.name_class == NC_OPERATOR)
+      {
+        const Function * fun = vs.sym_val.function;
+        if (fun == 0)
+           {
+             out << "⍝ function " << get_name() << " has function pointer 0!"
+                 << endl << endl;
+             return;
+           }
+
+        const UserFunction * ufun = fun->get_ufun1();
+        if (ufun == 0)
+           {
+             out << "⍝ function " << get_name() << " has ufun1 pointer 0!"
+                 << endl << endl;
+             return;
+           }
+
+        out << "∇";
+        const UCS_string text = ufun->canonical(false);
+        loop(u, text.size())
+           {
+              out << text[u];
+              if (text[u] == '\n' && u < text.size() - 1)   out << " ";
+           }
+        out << "∇" << endl << endl;
+      }
+}
+//-----------------------------------------------------------------------------
+void
 Symbol::clear_vs()
 {
    while (value_stack.size() > 1)
