@@ -122,7 +122,7 @@ SystemVariable::get_attributes(int mode, Cell * dest) const
 }
 //=============================================================================
 Quad_AV::Quad_AV()
-   : RO_SystemVariable(ID_QUAD_AV)
+   : RO_SystemVariable(ID_Quad_AV)
 {
    Symbol::assign(Value::AV_P, LOC);
 }
@@ -135,7 +135,7 @@ Quad_AV::indexed_at(uint32_t pos)
 }
 //=============================================================================
 Quad_AI::Quad_AI()
-   : RO_SystemVariable(ID_QUAD_AI),
+   : RO_SystemVariable(ID_Quad_AI),
      session_start(now()),
      user_wait(0)
 {
@@ -159,7 +159,7 @@ Value_P ret(new Value(4, LOC));
 }
 //=============================================================================
 Quad_ARG::Quad_ARG()
-   : RO_SystemVariable(ID_QUAD_ARG)
+   : RO_SystemVariable(ID_Quad_ARG)
 {
 }
 //-----------------------------------------------------------------------------
@@ -184,8 +184,8 @@ Cell * C = &Z->get_ravel(0);
 }
 //=============================================================================
 Quad_CT::Quad_CT()
-   : SystemVariable(ID_QUAD_CT),
-     current_ct(DEFAULT_QUAD_CT)
+   : SystemVariable(ID_Quad_CT),
+     current_ct(DEFAULT_Quad_CT)
 {
 Value_P value(new Value(LOC));
    new (&value->get_ravel(0)) FloatCell(current_ct);
@@ -212,7 +212,7 @@ APL_Float val = cell.get_real_value();
    if (val < 0.0)     DOMAIN_ERROR;
 
    // APL2 "discourages" the use of ⎕CT > 1E¯9.
-   // We round down to MAX_QUAD_CT (= 1E¯9) instead.
+   // We round down to MAX_Quad_CT (= 1E¯9) instead.
    //
    if (val > Value::Max_CT_P->get_ravel(0).get_real_value())
       {
@@ -281,7 +281,7 @@ Value_P Z(new Value(2, LOC));
    return Z;
 }
 //=============================================================================
-Quad_FC::Quad_FC() : SystemVariable(ID_QUAD_FC)
+Quad_FC::Quad_FC() : SystemVariable(ID_Quad_FC)
 {
    set_default();
 
@@ -332,9 +332,8 @@ ShapeItem value_len = value->element_count();
 }
 //-----------------------------------------------------------------------------
 void
-Quad_FC::assign_indexed(const IndexExpr & IX, Value_P value)
+Quad_FC::assign_indexed(IndexExpr & IX, Value_P value)
 {
-Q(LOC)
    if (IX.value_count() != 1)   INDEX_ERROR;
 
    // at this point we have a one dimensional index. It it were non-empty,
@@ -382,7 +381,7 @@ Value_P v1(new Value(6, LOC));
 }
 //=============================================================================
 Quad_IO::Quad_IO()
-   : SystemVariable(ID_QUAD_IO),
+   : SystemVariable(ID_Quad_IO),
      current_io(1)
 {
 Value_P value(new Value(LOC));
@@ -417,7 +416,7 @@ Quad_IO::expunge()
 }
 //=============================================================================
 Quad_L::Quad_L()
- : NL_SystemVariable(ID_QUAD_L)
+ : NL_SystemVariable(ID_Quad_L)
 {
    Symbol::assign(Value::Zero_P, LOC);
 }
@@ -449,7 +448,7 @@ const StateIndicator * si = Workspace::SI_top_error();
 }
 //=============================================================================
 Quad_LC::Quad_LC()
-   : RO_SystemVariable(ID_QUAD_LC)
+   : RO_SystemVariable(ID_Quad_LC)
 {
    Symbol::assign(Value::Zero_P, LOC);
 }
@@ -482,7 +481,7 @@ Value_P Z(new Value(len, LOC));
 }
 //=============================================================================
 Quad_LX::Quad_LX()
-   : NL_SystemVariable(ID_QUAD_LX)
+   : NL_SystemVariable(ID_Quad_LX)
 {
    Symbol::assign(Value::Str0_P, LOC);
 }
@@ -496,25 +495,65 @@ Quad_LX::assign(Value_P value, const char * loc)
    Symbol::assign(value, LOC);
 }
 //=============================================================================
-void
-Quad_NLT::assign(Value_P value, const char * loc)
+Quad_NLT::Quad_NLT()
+ : NL_SystemVariable(ID_Quad_NLT)
 {
-   if (value->get_rank() > 1)      RANK_ERROR;
-   if (!value->is_char_string())   DOMAIN_ERROR;
+   // get current locale
+   //
+const char * locale = setlocale(LC_CTYPE, 0);   // e,g, "en_US.utf8"
+   if (locale == 0)
+      {
+        Symbol::assign(Value::Str0_P, LOC);
+          return;
+      }
+
+UCS_string ulocale(locale);
+
+Value_P Z(new Value(ulocale, LOC));
+   Z->set_default(*Value::Spc_P);
+   Z->check_value(LOC);
+   Symbol::assign(Z, LOC);
+}
+//-----------------------------------------------------------------------------
+void
+Quad_NLT::assign(Value_P new_value, const char * loc)
+{
+   if (new_value->get_rank() > 1)      RANK_ERROR;
+   if (!new_value->is_char_string())   DOMAIN_ERROR;
 
    // expect language [_territory] [.code‐set] [@modifier]
    // e.g. en_US.utf8
    //
    // we only check that it is 7-bit ASCII and leave it to setlocale()
-   // to check if value is OK.
+   // to check if new_value is OK.
    //
-const ShapeItem len = value->element_count();
+const ShapeItem len = new_value->element_count();
    if (len > 100)   LENGTH_ERROR;
 
+   // setlocale() can mess up readline, so we want to use it only if new_value
+   // is different from the current value
+   //
+Value_P old_value = get_apl_value();
+   if (len == old_value->element_count())
+      {
+        bool same = true;
+        loop(l, len)
+           {
+             if (new_value->get_ravel(l).get_char_value() !=
+                 old_value->get_ravel(l).get_char_value())
+                {
+                  same = false;
+                  break;
+                }
+           }
+
+        if (same)   return;
+      }
+
 char new_locale[len + 1];
    loop(l, len)
       {
-         new_locale[l] = value->get_ravel(l).get_char_value();
+         new_locale[l] = new_value->get_ravel(l).get_char_value();
          if (new_locale[l] < ' ')    DOMAIN_ERROR;
          if (new_locale[l] > 0x7E)   DOMAIN_ERROR;
       }
@@ -524,30 +563,18 @@ char new_locale[len + 1];
 const char * locale = setlocale(LC_ALL, new_locale);
    if (locale == 0)   DOMAIN_ERROR;
 
-   // setlocale() has not complained, so we can set the ⎕NLT;
-   Symbol::assign(value, LOC);
-}
-//-----------------------------------------------------------------------------
-Value_P
-Quad_NLT::get_apl_value() const
-{
-   // get current locale
+   // setlocale() seems to mess up readline, so we need to re-init it
    //
-const char * locale = setlocale(LC_CTYPE, 0);   // e,g, "en_US.utf8"
-   if (locale == 0)   return Value::Str0_P;
+   Input::init(false);
 
-UCS_string ulocale(locale);
-
-Value_P Z(new Value(ulocale, LOC));
-   Z->set_default(*Value::Spc_P);
-
-   Z->check_value(LOC);
-   return Z;
+   // setlocale() has not complained, so we can set the ⎕NLT;
+   //
+   Symbol::assign(new_value, LOC);
 }
 //=============================================================================
 Quad_PP::Quad_PP()
-   : SystemVariable(ID_QUAD_PP),
-     current_pp(DEFAULT_QUAD_PP)
+   : SystemVariable(ID_Quad_PP),
+     current_pp(DEFAULT_Quad_PP)
 {
 Value_P value(new Value(LOC));
 
@@ -568,11 +595,11 @@ Quad_PP::assign(Value_P value, const char * loc)
 
 const Cell & cell = value->get_ravel(0);
 const APL_Integer val = cell.get_near_int(0.1);
-   if (val < MIN_QUAD_PP)   DOMAIN_ERROR;
+   if (val < MIN_Quad_PP)   DOMAIN_ERROR;
 
-   if (val > MAX_QUAD_PP)
+   if (val > MAX_Quad_PP)
       {
-        current_pp = MAX_QUAD_PP;
+        current_pp = MAX_Quad_PP;
         value = Value::Max_PP_P;
       }
    else
@@ -584,7 +611,7 @@ const APL_Integer val = cell.get_near_int(0.1);
 }
 //=============================================================================
 Quad_PR::Quad_PR()
-   : SystemVariable(ID_QUAD_PR),
+   : SystemVariable(ID_Quad_PR),
      current_pr(UNI_ASCII_SPACE)
 {
 Value_P value(new Value(current_pr, LOC));
@@ -606,7 +633,7 @@ UCS_string ucs = value->get_UCS_ravel();
 }
 //=============================================================================
 Quad_PS::Quad_PS()
-   : SystemVariable(ID_QUAD_PS),
+   : SystemVariable(ID_Quad_PS),
      current_ps(PR_APL)
 {
    Symbol::assign(Value::Zero_P, LOC);
@@ -634,7 +661,7 @@ const APL_Integer val = cell.get_near_int(0.1);
 }
 //=============================================================================
 Quad_PT::Quad_PT()
-   : RO_SystemVariable(ID_QUAD_PT)
+   : RO_SystemVariable(ID_Quad_PT)
 {
    Symbol::assign(Value::Zero_P, LOC);
 }
@@ -650,7 +677,7 @@ Value_P Z(new Value(LOC));
 }
 //=============================================================================
 Quad_PW::Quad_PW()
-   : SystemVariable(ID_QUAD_PW),
+   : SystemVariable(ID_Quad_PW),
      current_pw(80)
 {
 Value_P value(new Value(LOC));
@@ -677,19 +704,19 @@ const APL_Integer val = cell.get_near_int(0.1);
    if (val < 30)   return;
 
    // max val is system specific. Ignore larger values.
-   if (val > MAX_QUAD_PW)   return;
+   if (val > MAX_Quad_PW)   return;
 
    current_pw = val;
    Symbol::assign(value, LOC);
 }
 //=============================================================================
-Quad_QUAD::Quad_QUAD()
- : SystemVariable(ID_QUAD_QUAD)
+Quad_Quad::Quad_Quad()
+ : SystemVariable(ID_Quad_Quad)
 {
 }
 //-----------------------------------------------------------------------------
 void
-Quad_QUAD::assign(Value_P value, const char * loc)
+Quad_Quad::assign(Value_P value, const char * loc)
 {
    // write pending LF from  ⍞ (if any)
    Quad_QUOTE::done(true, LOC);
@@ -698,14 +725,14 @@ Quad_QUAD::assign(Value_P value, const char * loc)
 }
 //-----------------------------------------------------------------------------
 void
-Quad_QUAD::resolve(Token & token, bool left)
+Quad_Quad::resolve(Token & token, bool left)
 {
    if (left)   return;   // ⎕←xxx
 
    // write pending LF from  ⍞ (if any)
    Quad_QUOTE::done(true, LOC);
 
-   COUT << UNI_QUAD_QUAD << ":" << endl;
+   COUT << UNI_Quad_Quad << ":" << endl;
 
 UCS_string in = Input::get_line();
 
@@ -713,7 +740,7 @@ UCS_string in = Input::get_line();
 }
 //=============================================================================
 Quad_QUOTE::Quad_QUOTE()
- : SystemVariable(ID_QUOTE_QUAD)
+ : SystemVariable(ID_QUOTE_Quad)
 {
    // we assign a dummy value so that ⍞ is not undefined.
    //
@@ -744,7 +771,7 @@ Quad_QUOTE::assign(Value_P value, const char * loc)
       CERR << "Quad_QUOTE::assign() called, buffer = ["
            << prompt << "]" << endl;
 
-PrintContext pctx(PR_QUOTE_QUAD);
+PrintContext pctx(PR_QUOTE_Quad);
 PrintBuffer pb(*value, pctx);
    if (pb.get_height() > 1)   // multi line output: flush and restart corking
       {
@@ -757,7 +784,7 @@ PrintBuffer pb(*value, pctx);
       }
    else
       {
-        COUT << pb.l1();
+        COUT << pb.l1().no_pad();
         prompt.append(pb.l1());
       }
 
@@ -796,7 +823,7 @@ Value_P Z(new Value(in, LOC));
 }
 //=============================================================================
 Quad_R::Quad_R()
- : NL_SystemVariable(ID_QUAD_R)
+ : NL_SystemVariable(ID_Quad_R)
 {
    Symbol::assign(Value::Zero_P, LOC);
 }
@@ -842,15 +869,15 @@ Quad_SYL::assign(Value_P value, const char * loc)
 }
 //-----------------------------------------------------------------------------
 void
-Quad_SYL::assign_indexed(const IndexExpr & IDX, Value_P value)
+Quad_SYL::assign_indexed(IndexExpr & IDX, Value_P value)
 {
-   //  must be an array index of the form [2; something]
+   //  must be an array index of the form [something; 2]
    //
    if (IDX.value_count() != 2)   INDEX_ERROR;
 
    // IDX is in reverse order: ⎕SYL[X1;X2]
    //
-Value_P X2 = IDX.values[0];
+Value_P X2 = IDX.extract_value(0);
 
 const APL_Float qct = Workspace::get_CT();
 const APL_Integer qio = Workspace::get_IO();
@@ -860,8 +887,7 @@ const APL_Integer qio = Workspace::get_IO();
    if (!X2->get_ravel(0).is_near_int(qct))              INDEX_ERROR;
    if (X2->get_ravel(0).get_near_int(qct) != qio + 1)   INDEX_ERROR;
 
-Value_P X1 = IDX.values[1];
-   delete &IDX;
+Value_P X1 = IDX.extract_value(1);
 
    assign_indexed(X1, value);
 }
@@ -886,18 +912,20 @@ const APL_Integer qio = Workspace::get_IO();
         if (x == SYL_SI_DEPTH_LIMIT)   // SI depth limit
            {
              const int depth = Workspace::SI_entry_count();
-             if (b < (depth + 4))   DOMAIN_ERROR;    // limit too low
+             if (b && (b < (depth + 4)))   DOMAIN_ERROR;    // limit too low
              si_depth_limit = b;
            }
         else if (x == SYL_VALUE_COUNT_LIMIT)   // value count limit
            {
-             if (b < (Value::value_count + 10))   DOMAIN_ERROR;    // too low
+             if (b && (b < (Value::value_count + 10)))    // too low
+                DOMAIN_ERROR;
              value_count_limit = b;
            }
-        else if (x == SYL_RAVEL_BYTES_LIMIT)   // ravel bytes depth limit
+        else if (x == SYL_RAVEL_BYTES_LIMIT)   // ravel bytes limit
            {
              const int cells = b / sizeof(Cell);
-             if (cells < (Value::total_ravel_count + 1000))   DOMAIN_ERROR;
+             if (cells && (cells < (Value::total_ravel_count + 1000)))
+                DOMAIN_ERROR;
 
              ravel_count_limit = cells;
            }
@@ -925,7 +953,7 @@ Value_P Z( new Value(sh, LOC));
 }
 //=============================================================================
 Quad_TC::Quad_TC()
-   : RO_SystemVariable(ID_QUAD_TC)
+   : RO_SystemVariable(ID_Quad_TC)
 {
    Symbol::assign(Value::Quad_TC_P, LOC);
 }
@@ -937,7 +965,7 @@ Quad_TC::get_apl_value() const
 }
 //=============================================================================
 Quad_TS::Quad_TS()
-   : RO_SystemVariable(ID_QUAD_TS)
+   : RO_SystemVariable(ID_Quad_TS)
 {
    Symbol::assign(Value::Zero_P, LOC);
 }
@@ -945,8 +973,8 @@ Quad_TS::Quad_TS()
 Value_P
 Quad_TS::get_apl_value() const
 {
-const int offset = Workspace::get_v_quad_TZ().get_offset();
-const YMDhmsu time(now() + offset);
+const APL_time_us offset_us = 1000000 * Workspace::get_v_Quad_TZ().get_offset();
+const YMDhmsu time(now() + offset_us);
 
 Value_P Z(new Value(7, LOC));
    new (&Z->get_ravel(0)) IntCell(time.year);
@@ -962,27 +990,37 @@ Value_P Z(new Value(7, LOC));
 }
 //=============================================================================
 Quad_TZ::Quad_TZ()
-   : SystemVariable(ID_QUAD_TZ)
+   : SystemVariable(ID_Quad_TZ)
 {
    // the GNU/Linux timezone variable conflicts with function timezone() on
    // other systems. timezone.tz_minuteswest in gettimeofday() does not
    // sound reliable either.
    // 
    // We compute localtime() and gmtime() and take the difference (hoping
-   // that localtime() and gmtime() are mor portable.
+   // that localtime() and gmtime() are more portable.
    // 
-timeval now;
-   gettimeofday(&now, 0);
 
-tm * local = localtime(&now.tv_sec);
+   // choose a reference point in time that is the same month in all time zones.
+   // we use Monday Jan 2014 in the afternoon
+   //
+const time_t ref = now()/1000000;
+
+tm * local = localtime(&ref);
 const int local_minutes = local->tm_min + 60*local->tm_hour;
+const int local_days = local->tm_mday + 31*local->tm_mon + 12*31*local->tm_year;
+
+tm * gmean = gmtime(&ref);
+const int gm_minutes    = gmean->tm_min + 60*gmean->tm_hour;
+const int gm_days = gmean->tm_mday + 31*gmean->tm_mon + 12*31*gmean->tm_year;
+int diff_minutes = local_minutes - gm_minutes;
+
+   if (local_days < gm_days)        diff_minutes -= 24*60;   // local < gmean
+   else if (local_days > gm_days)   diff_minutes += 24*60;   // local > gmean
 
-tm * gmean = gmtime(&now.tv_sec);
-     const int gm_minutes = gmean->tm_min + 60*gmean->tm_hour;
-     
-const int diff_minutes = local_minutes - gm_minutes;
    offset_seconds = 60*diff_minutes;
 
+   // ⎕TZ is the offset in hours between GMT and local time
+   //
 Value_P value(new Value(LOC));
    if (offset_seconds % 3600 == 0)   // full hour
       new (&value->get_ravel(0))   IntCell(offset_seconds/3600);
@@ -999,14 +1037,14 @@ Quad_TZ::assign(Value_P value, const char * loc)
 {
    if (!value->is_skalar())   RANK_ERROR;
 
-   // ignore values outside [-12 ... 12], DOMAIN ERROR for bad types.
+   // ignore values outside [-12 ... 14], DOMAIN ERROR for bad types.
 
 Cell & cell = value->get_ravel(0);
    if (cell.is_integer_cell())
       {
         const APL_Integer ival = cell.get_int_value();
         if (ival < -12)   return;
-        if (ival > 12)    return;
+        if (ival > 14)    return;
         offset_seconds = ival*3600;
         Symbol::assign(value, LOC);
         return;
@@ -1015,8 +1053,8 @@ Cell & cell = value->get_ravel(0);
    if (cell.is_float_cell())
       {
         const APL_Float fval = cell.get_real_value();
-        if (fval < -12)   return;
-        if (fval > 12)    return;
+        if (fval < -12.1)   return;
+        if (fval > 14.1)    return;
         offset_seconds = int(0.5 + fval*3600);
         Symbol::assign(value, LOC);
         return;
@@ -1026,7 +1064,7 @@ Cell & cell = value->get_ravel(0);
 }
 //=============================================================================
 Quad_UL::Quad_UL()
-   : RO_SystemVariable(ID_QUAD_UL)
+   : RO_SystemVariable(ID_Quad_UL)
 {
    Symbol::assign(get_apl_value(), LOC);
 }
@@ -1059,7 +1097,7 @@ Value_P Z(new Value(LOC));
 }
 //=============================================================================
 Quad_WA::Quad_WA()
-   : RO_SystemVariable(ID_QUAD_WA)
+   : RO_SystemVariable(ID_Quad_WA)
 {
    Symbol::assign(Value::Zero_P, LOC);
 }
