@@ -72,7 +72,7 @@ Tokenizer::do_tokenize(const UCS_string & input, Token_string & tos)
            << input << "»" << endl;
 
 Source<Unicode> src(input);
-   while (rest_1 = rest_2 = src.rest())
+   while ((rest_1 = rest_2 = src.rest()) != 0)
       {
         const Unicode uni = *src;
         if (uni == UNI_COMMENT)   break;
@@ -113,11 +113,47 @@ Source<Unicode> src(input);
                       {
                          tokenize_quad(src, tos);
                       }
-                   else if (uni == UNI_QUOTE_QUAD)
+                   else if (uni == UNI_QUOTE_Quad)
                       {
                         ++src;
-                        tos.append(Token(TOK_QUAD_QUOTE,
-                                   &Workspace::get_v_quad_QUOTE()), LOC);
+                        tos.append(Token(TOK_Quad_QUOTE,
+                                   &Workspace::get_v_Quad_QUOTE()), LOC);
+                      }
+                   else if (uni == UNI_ALPHA)
+                      {
+                        ++src;
+                        tos.append(Token(TOK_ALPHA,
+                                   &Workspace::get_v_ALPHA()), LOC);
+                      }
+                   else if (uni == UNI_ALPHA_UNDERBAR)
+                      {
+                        ++src;
+                        tos.append(Token(TOK_ALPHA_U,
+                                   &Workspace::get_v_ALPHA_U()), LOC);
+                      }
+                   else if (uni == UNI_CHI)
+                      {
+                        ++src;
+                        tos.append(Token(TOK_CHI,
+                                   &Workspace::get_v_CHI()), LOC);
+                      }
+                   else if (uni == UNI_LAMBDA)
+                      {
+                        ++src;
+                        tos.append(Token(TOK_LAMBDA,
+                                   &Workspace::get_v_LAMBDA()), LOC);
+                      }
+                   else if (uni == UNI_OMEGA)
+                      {
+                        ++src;
+                        tos.append(Token(TOK_OMEGA,
+                                   &Workspace::get_v_OMEGA()), LOC);
+                      }
+                   else if (uni == UNI_OMEGA_UNDERBAR)
+                      {
+                        ++src;
+                        tos.append(Token(TOK_OMEGA_U,
+                                   &Workspace::get_v_OMEGA_U()), LOC);
                       }
                    else
                       {
@@ -167,6 +203,8 @@ Source<Unicode> src(input);
               case TC_R_PARENT:
               case TC_L_BRACK:
               case TC_R_BRACK:
+              case TC_L_CURLY:
+              case TC_R_CURLY:
                    ++src;
                    tos.append(tok, LOC);
                    break;
@@ -263,6 +301,7 @@ Token tok = Avec::uni_to_token(uni, LOC);
         sys(F12_EQUIV,     F12_EQUIV)
         sys(F12_FORMAT,    F12_FORMAT)
         sys(F12_INDEX_OF,  F12_INDEX_OF)
+        sys(F2_INTER,      F2_INTER)
         sys(F2_LEFT,       F2_LEFT)
         sys(F12_LOGA,      F12_LOGA)
         sys(F12_MINUS,     F12_MINUS)
@@ -283,7 +322,7 @@ Token tok = Avec::uni_to_token(uni, LOC);
         sys(F12_TAKE,      F12_TAKE)
         sys(F12_TRANSPOSE, F12_TRANSPOSE)
         sys(F12_TIMES,     F12_TIMES)
-        sys(F1_UNIQUE,     F1_UNIQUE)
+        sys(F12_UNION,     F12_UNION)
         sys(F12_WITHOUT,   F12_WITHOUT)
 
         sys(JOT,           JOT)
@@ -312,14 +351,20 @@ Tokenizer::tokenize_quad(Source<Unicode> & src, Token_string & tos)
    Log(LOG_tokenize)
       CERR << "tokenize_quad(" << src.rest() << " chars)"<< endl;
 
-UCS_string ucs;
-   ucs.append(src.get());
-   Assert((ucs[0]));
+   src.get();               // discard (possibly alternative) ⎕
+UCS_string ucs(UNI_Quad_Quad);
+   Assert(ucs[0]);
 
    ucs.append((src.rest() > 0) ? src[0] : Invalid_Unicode);
    ucs.append((src.rest() > 1) ? src[1] : Invalid_Unicode);
    ucs.append((src.rest() > 2) ? src[2] : Invalid_Unicode);
 
+   // allow lowercase ⎕xx
+   //
+   if (ucs[1] >= 'a' && ucs[1] <= 'z')   ucs[1] = (Unicode)(ucs[1] - 32);
+   if (ucs[2] >= 'a' && ucs[2] <= 'z')   ucs[2] = (Unicode)(ucs[2] - 32);
+   if (ucs[3] >= 'a' && ucs[3] <= 'z')   ucs[3] = (Unicode)(ucs[3] - 32);
+
 int len = 0;
 const Token t = Workspace::get_quad(ucs, len);
    src.skip(len - 1);
@@ -466,8 +511,8 @@ Tokenizer::tokenize_number(Source<Unicode> & src, Token_string & tos)
    // real 'D' real   // magnitude + angle in degrees
    // real 'R' real   // magnitude + angle in radian
 
-APL_Float real_flt;   // always valid
-APL_Integer real_int;   // valid if real_floating is true
+APL_Float real_flt   = 0.0;   // always valid
+APL_Integer real_int = 0.0;   // valid if real_floating is true
 const bool real_valid = tokenize_real(src, real_flt, real_int);
 bool real_floating = real_flt != real_int;
 
@@ -552,15 +597,15 @@ Tokenizer::tokenize_real(Source<Unicode> &src, APL_Float & flt_val,
 const int32_t src_size = src.rest();
 bool dot_seen = false;
 int E_pos = 0;   // position of 'e' in utf
-int minus_pos = 0;
+int minus_pos = 0;   // position of (exponent-) ¯ in utf
 bool digit_seen = false;
 
    // copy chars from src to utf
    //
 UTF8_string utf;
 
-   // a '-' can occur in the mantissa and in the exponent. To make a '-'
-   // unique, we handle a leading (i.e. mantissa) '-' beforehand.
+   // a '¯' can occur in the mantissa and in the exponent. To make a '¯'
+   // unique, we handle a leading (i.e. mantissa) '¯' beforehand.
    //
    if (*src == UNI_OVERBAR)
       {
@@ -603,7 +648,7 @@ UTF8_string utf;
            }
       }
 
-   // E or E¯ without exponent digits
+   // handle E or E¯ without exponent digits
    //
    if (utf.size() == E_pos)       return false;
    if (utf.size() == minus_pos)   return false;
@@ -619,6 +664,84 @@ UTF8_string utf;
    return false;
 }
 //-----------------------------------------------------------------------------
+#if 0
+int
+Tokenizer::scan_real(const char * strg, double & result,
+                     int E_pos, int minus_pos)
+{
+   // this function (which is currently not used) shall become a locale
+   // independent sscanf. Currently it gives slightly different results
+   // than sscanf (rounding errors?) and therefore breaks a few testcases.
+   // TODO!
+   //
+const bool negative = *strg == '-';
+int p = 0;
+int expo_f = 0;
+int expo_E = 0;
+
+   if (negative)   ++p;   // leading -
+
+   result = 0;
+
+   // the caller has filtered everything but 0-9 . - and e.
+   //
+
+   // integer part
+   //
+   for (;;)
+      {
+        const char cc = strg[p++];
+        if (cc == 0)     goto done;
+        if (cc == '.')   goto fract;
+        if (cc == 'e')   goto expo;
+        result *= 10;
+        result += cc - '0';
+      }
+
+fract:
+   for (;;)
+      {
+        const char cc = strg[p++];
+        if (cc == 0)     goto done;
+        if (cc == 'e')   goto expo;
+        result *= 10;
+        result += cc - '0';
+        --expo_f;
+      }
+
+expo:
+   {
+     if (minus_pos)   p = minus_pos;
+     else             p = E_pos;
+
+   for (;;)
+      {
+        const char cc = strg[p++];
+        if (cc == 0)     break;
+        expo_E *= 10;
+        expo_E += cc - '0';
+      }
+   }
+
+done:
+     if (minus_pos)   expo_E = -expo_E;
+     expo_E += expo_f;
+
+     if (expo_E)
+        {
+          if (expo_E <= -308)
+             {
+               result = 0;
+               return 1;
+             }
+          result *= exp10(expo_E);
+        }
+
+   if (negative)   result = -result;
+   return 1;
+}
+#endif
+//-----------------------------------------------------------------------------
 void
 Tokenizer::tokenize_symbol(Source<Unicode> & src, Token_string & tos)
 {
