@@ -61,29 +61,30 @@ UCS_string::UCS_string(const UTF8_string & utf)
         else if ((b0 & 0xFE) == 0xFC)   { more = 5; bx &= 0x03; }
         else
            {
-             CERR << "Bad UTF8 sequence: " << HEX(b0);
-             loop(j, 6)
-                {
-                  if ((i + j) >= utf.size())   break;
-                  const uint32_t bx = utf[i + j];
-                  if (bx & 0x80)   CERR << " " << HEX(bx);
-                  else             break;
-
-                }
-             CERR << endl;
+             utf.dump_hex(CERR << "Bad UTF8 string: ", 40)
+                               << " at " << LOC <<  endl;
              Backtrace::show(__FILE__, __LINE__);
-             Assert(0 && "Internal error in UCS_string::UCS_string()");
+             Assert(0 && "Error in UCS_string::UCS_string()");
              break;
            }
 
         uint32_t uni = 0;
         for (; more; --more)
             {
+              if (i >= utf.size())
+                 {
+                   utf.dump_hex(CERR << "Truncated UTF8 string: ", 40)
+                      << " at " << LOC <<  endl;
+                   return;
+                 }
+
               const UTF8 subc = utf[i++];
               if ((subc & 0xC0) != 0x80)
                  {
-                   CERR << "Bad UTF8 sequence: " << HEX(b0) << "..." << endl;
-                   Assert(0 && "Internal error in UCS_string::UCS_string()");
+                   utf.dump_hex(CERR << "Bad UTF8 string: ", 40)
+                      << " at " << LOC <<  endl;
+                   Backtrace::show(__FILE__, __LINE__);
+                   Assert(0 && "Error in UCS_string::UCS_string()");
                  }
 
               bx  <<= 6;
@@ -103,8 +104,8 @@ UCS_string::UCS_string(APL_Float value, bool & scaled,
    : Simple_string<Unicode>(0, 0)
 {
 int quad_pp = pctx.get_PP();
-   if (quad_pp > MAX_QUAD_PP)   quad_pp = MAX_QUAD_PP;
-   if (quad_pp < MIN_QUAD_PP)   quad_pp = MIN_QUAD_PP;
+   if (quad_pp > MAX_Quad_PP)   quad_pp = MAX_Quad_PP;
+   if (quad_pp < MIN_Quad_PP)   quad_pp = MIN_Quad_PP;
 
 const bool negative = (value < 0);
    if (negative)   value = -value;
@@ -116,8 +117,9 @@ int expo = 0;
        while (value >= 1e16)
              {
                if (value > 1e307)
-                  { append(negative ? UNI_ASCII_MINUS : UNI_ASCII_PLUS);
-                    append(UNI_INFINITY);
+                  {
+                    if (negative)   append_utf8("¯∞");
+                    else            append_utf8("∞");
                     FloatCell::map_FC(*this);
                     return;
                   }
@@ -269,27 +271,103 @@ const Unicode last = digits.last();
    FloatCell::map_FC(*this);
 }
 //-----------------------------------------------------------------------------
-UCS_string::UCS_string(const PrintBuffer & pb, Rank rank)
+UCS_string::UCS_string(const PrintBuffer & pb, Rank rank, int quad_PW)
    : Simple_string<Unicode>(0, 0)
 {
-   if (pb.get_height() == 0)   return;
+   if (pb.get_height() == 0)   return;      // empty PrintBuffer
 
-   if (pb.get_break_points().size() == 0)   // not chopped by ⎕PW
-      {
-        add_chunk(pb, 0, pb.get_width(0));
-        return;
-      }
+const int pb_width = pb.get_width(0);
+vector<int> breakpoints;
 
-int col = 0;
-   loop(b, pb.get_break_points().size())
+   // compute breakpoints
+   //
+   for (int col = 0; col < pb_width;)
        {
-         const int brkp = pb.get_break_points()[b];
-         add_chunk(pb, col, brkp - col);
-         col = brkp;
-         loop(r, rank)   append(UNI_ASCII_LF);
+         const int max_chunk_width = col ? quad_PW - 6 : quad_PW;
+
+         // find breakpoint
+         //
+         int chunk_len = max_chunk_width;
+         const int rest = pb_width - col;
+         if (chunk_len < rest)   // rest too large
+            {
+              // search backwards for break char
+              //
+              int pos = col + chunk_len;
+              if (pos > pb_width)   pos = pb_width;
+              while (--pos > col)
+                  {
+                    const Unicode uni = pb.get_line(0)[pos];
+                    if (uni == UNI_iPAD_U2 || uni == UNI_iPAD_U3)
+                       {
+                          chunk_len = pos - col + 1;
+                          break;
+                       }
+                  }
+            }
+
+         breakpoints.push_back(chunk_len);
+         col += chunk_len;
        }
 
-   add_chunk(pb, col, pb.get_width(0) - col);
+   // print rows, breaking at breakpoints
+   //
+   loop(row, pb.get_height())
+       {
+         if (row)   append(UNI_ASCII_LF);   // end previous row
+         int col = 0;
+         loop(b, breakpoints.size())
+            {
+              const int chunk_len = breakpoints[b];
+              if (col)   append_utf8("\n      ");
+              UCS_string trow(pb.get_line(row), col, chunk_len);
+              trow.remove_trailing_padchars();
+              append(trow);
+              col += chunk_len;
+            }
+       }
+
+   // replacing pad chars with blanks.
+   //
+   loop(u, size())   if (is_iPAD_char((*this)[u]))   (*this)[u] = UNI_ASCII_SPACE;
+}
+//-----------------------------------------------------------------------------
+void
+UCS_string::remove_trailing_padchars()
+{
+   // remove trailing pad chars from align() and append_string(),
+   // but leave other pad chars intact.
+   // But only if the line has no frame (vert).
+   //
+
+   // If the line contains UNI_iPAD_L0 (higher dimension separator)
+   // then discard all chars.
+   //
+   loop(u, size())
+       {
+         if ((*this)[u] == UNI_LINE_VERT)    break;
+         if ((*this)[u] == UNI_LINE_VERT2)   break;
+         if ((*this)[u] == UNI_iPAD_L0)
+            {
+              clear();
+              return;
+            }
+
+       }
+
+   while (size())
+      {
+        const Unicode last = back();
+        if (last == UNI_iPAD_L0 ||
+            last == UNI_iPAD_L1 ||
+            last == UNI_iPAD_L2 ||
+            last == UNI_iPAD_L3 ||
+            last == UNI_iPAD_L4 ||
+            last == UNI_iPAD_U7)
+            pop_back();
+        else
+            break;
+      }
 }
 //-----------------------------------------------------------------------------
 /// constructor
@@ -317,63 +395,6 @@ UCS_string::UCS_string(istream & in)
 }
 //-----------------------------------------------------------------------------
 void
-UCS_string::add_chunk(const PrintBuffer & pb, int from, size_t width)
-{
-   loop(y, pb.get_height())
-       {
-         if (y)   append(UNI_ASCII_LF);
-
-         UCS_string ucs;
-         if (from)   ucs.append(UCS_string(UTF8_string("      ")));
-         ucs.append(UCS_string (pb.get_line(y), from, width));
-
-         // remove trailing pad chars from align() and append_string(),
-         // but leave other pad chars intact.
-         // But only if the line has no frame (vert).
-         //
-         {
-           // If the line contains UNI_PAD_L0 (higher dimension separator)
-           // then discard all chars.
-           //
-           bool has_L0 = false;
-           loop(u, ucs.size())
-               {
-                 if (ucs[u] == UNI_LINE_VERT)    goto non_empty;
-                 if (ucs[u] == UNI_LINE_VERT2)   goto non_empty;
-                 if (ucs[u] == UNI_PAD_L0)       { has_L0 = true;   break; }
-
-               }
-
-           if (has_L0)   continue;   // next line
-         }
-
-         non_empty:
-
-         while (ucs.size() > 0)
-            {
-              const Unicode last = ucs.back();
-              if (last == UNI_PAD_L0 ||
-                  last == UNI_PAD_L1 ||
-                  last == UNI_PAD_L2 ||
-                  last == UNI_PAD_L3 ||
-                  last == UNI_PAD_L4 ||
-                  last == UNI_PAD_U7)
-                 ucs.pop_back();
-              else
-                 break;
-            }
-
-         // print the value, replacing pad chars with blanks.
-         loop(u, ucs.size())
-            {
-              Unicode uni = ucs[u];
-              if (is_pad_char(uni))   uni = UNI_ASCII_SPACE;
-              append(uni);
-            }
-       }
-}
-//-----------------------------------------------------------------------------
-void
 UCS_string::copy_black(UCS_string & dest, int & idx) const
 {
    while (idx < size() && operator[](idx) <= ' ')   ++idx;
@@ -469,7 +490,7 @@ UCS_string ret;
    loop(s, size())
       {
         Unicode uni = (*this)[s];
-        if (is_pad_char(uni))   uni = UNI_ASCII_SPACE;
+        if (is_iPAD_char(uni))   uni = UNI_ASCII_SPACE;
         ret.append(uni);
       }
 
@@ -483,7 +504,7 @@ UCS_string ret;
    loop(s, size())
       {
         Unicode uni = (*this)[s];
-        if (!is_pad_char(uni))   ret.append(uni);
+        if (!is_iPAD_char(uni))   ret.append(uni);
       }
 
    return ret;
@@ -624,28 +645,36 @@ operator << (ostream & os, Unicode uni)
 {       
    if (uni < 0x80)      return os << (char)uni;
         
-   if (uni < 0x800)     return os << (char)(0xC0 | uni >> 6)
-                                  << (char)(0x80 | uni & 0x3F);
+   if (uni < 0x800)     return os << (char)(0xC0 | (uni >> 6))
+                                  << (char)(0x80 | (uni & 0x3F));
         
-   if (uni < 0x10000)    return os << (char)(0xE0 | uni >> 12)
-                                   << (char)(0x80 | uni >>  6 & 0x3F)
-                                   << (char)(0x80 | uni       & 0x3F);
+   if (uni < 0x10000)    return os << (char)(0xE0 | (uni >> 12))
+                                   << (char)(0x80 | (uni >>  6 & 0x3F))
+                                   << (char)(0x80 | (uni       & 0x3F));
 
-   if (uni < 0x110000)   return os << (char)(0xE0 | uni >> 18)
-                                   << (char)(0x80 | uni >> 12 & 0x3F)
-                                   << (char)(0x80 | uni >>  6 & 0x3F)
-                                   << (char)(0x80 | uni       & 0x3F);
+   if (uni < 0x200000)   return os << (char)(0xF0 | (uni >> 18))
+                                   << (char)(0x80 | (uni >> 12 & 0x3F))
+                                   << (char)(0x80 | (uni >>  6 & 0x3F))
+                                   << (char)(0x80 | (uni       & 0x3F));
 
-   // use ѡ to display invalid unicodes
-   return os << (char)0xD1 << (char)0xA1;
-//   return os << UNI(uni);
+   if (uni < 0x4000000)  return os << (char)(0xF8 | (uni >> 24))
+                                   << (char)(0x80 | (uni >> 18 & 0x3F))
+                                   << (char)(0x80 | (uni >> 12 & 0x3F))
+                                   << (char)(0x80 | (uni >>  6 & 0x3F))
+                                   << (char)(0x80 | (uni       & 0x3F));
+
+   return os << (char)(0xFC | (uni >> 30))
+             << (char)(0x80 | (uni >> 24 & 0x3F))
+             << (char)(0x80 | (uni >> 18 & 0x3F))
+             << (char)(0x80 | (uni >> 12 & 0x3F))
+             << (char)(0x80 | (uni >>  6 & 0x3F))
+             << (char)(0x80 | (uni       & 0x3F));
 }
 //-----------------------------------------------------------------------------
 ostream &
 operator << (ostream & os, const UCS_string & ucs)
 {
-UTF8_string utf(ucs);
-   os << utf.c_str();
+   loop(u, ucs.size())   os << ucs[u];
    return os;
 }
 //-----------------------------------------------------------------------------
