@@ -120,7 +120,11 @@ UTF8_string::UTF8_string(const UCS_string & ucs)
            }
         else
            {
-             CERR << "Bad Unicode: " << UNI(uni) << endl;
+             CERR << "Bad Unicode: " << UNI(uni) << endl
+                  << "The offending ucs string is:";
+             loop(ii, ucs.size()) CERR << " " << HEX(ucs[ii]);
+             CERR << endl;
+
              Backtrace::show(__FILE__, __LINE__);
              Assert(0 && "Error in UTF8_string::UTF8_string(ucs)");
            }
@@ -136,6 +140,20 @@ UCS_string ucs(value);
    new (this) UTF8_string(ucs);
 }
 //-----------------------------------------------------------------------------
+ostream &
+UTF8_string::dump_hex(ostream & out, int max_bytes) const
+{
+   loop(b, size())
+      {
+        if (b)   out << " ";
+        if (b >= max_bytes)   return out << "...";
+
+         out << HEX2((*this)[b]);
+      }
+
+   return out;
+}
+//-----------------------------------------------------------------------------
 Unicode
 UTF8_string::toUni(const UTF8 * string, int & len)
 {
