@@ -100,7 +100,8 @@ const ShapeItem length = shape.get_volume();
 
         try
            {
-             ravel = new Cell[length];
+             Cell * long_ravel =  new Cell[length];
+             ravel = long_ravel;
            }
         catch (...)
            {
@@ -211,12 +212,12 @@ Value::Value(const char * loc, Value_how how)
 
         case Value_how_Max_CT:
              init_ravel();
-             new (&get_ravel(0)) FloatCell(MAX_QUAD_CT);
+             new (&get_ravel(0)) FloatCell(MAX_Quad_CT);
              break;
 
         case Value_how_Max_PP:
              init_ravel();
-             new (&get_ravel(0)) IntCell(MAX_QUAD_PP);
+             new (&get_ravel(0)) IntCell(MAX_Quad_PP);
              break;
 
         case Value_how_Quad_NLT:
@@ -344,7 +345,7 @@ const ShapeItem length = nz_element_count();
    if (ravel != short_value)
       {
         total_ravel_count -= length;
-        delete ravel;
+        delete [] ravel;
       }
 
    Assert(check_ptr == (const char *)this + 7);
@@ -364,7 +365,6 @@ Value_P
 Value::get_cellrefs(const char * loc)
 {
 Value_P ret(new Value(get_shape(), loc));
-   ret->set_left();
 
 const ShapeItem ec = nz_element_count();
 
@@ -405,6 +405,7 @@ const int src_incr = new_value->is_skalar() ? 0 : 1;
         return;
       }
 
+   if (value_count != dest_count)   LENGTH_ERROR;
 
    loop(d, dest_count)
       {
@@ -626,7 +627,6 @@ int count = 0;
               v->list_one(CERR, false);
             }
 
-         v->set_dirty();
          ++count;
        }
 
@@ -641,10 +641,9 @@ Value::list_one(ostream & out, bool show_owners)
         out << "   Flags =";
         char sep = ' ';
         if (is_forever())    { out << sep << "FOREVER";    sep = '+'; }
-        if (is_left())       { out << sep << "LEFT";       sep = '+'; }
-        if (is_dirty())      { out << sep << "DIRTY";      sep = '+'; }
         if (is_complete())   { out << sep << "COMPLETE";   sep = '+'; }
         if (is_marked())     { out << sep << "MARKED";     sep = '+'; }
+        if (is_temp())       { out << sep << "TEMP";       sep = '+'; }
         out << ",";
       }
    else
@@ -874,6 +873,23 @@ const ShapeItem ec = nz_element_count();
    return false;   // all cells numeric and not complex
 }
 //-----------------------------------------------------------------------------
+bool
+Value::is_simple() const
+{
+const ShapeItem count = element_count();
+const Cell * C = &get_ravel(0);
+
+
+   loop(c, count)
+       {
+         if (C->is_pointer_cell())   return false;
+         if (C->is_lval_cell())      return false;
+         ++C;
+       }
+
+   return true;
+}
+//-----------------------------------------------------------------------------
 Depth
 Value::compute_depth() const
 {
@@ -888,7 +904,7 @@ Value::compute_depth() const
         return 0;
       }
 
-ShapeItem count = nz_element_count();
+const ShapeItem count = nz_element_count();
 
 Depth sub_depth = 0;
    loop(c, count)
@@ -965,6 +981,10 @@ Shape shape_Z;
             }
        }
 
+   // check that all indices are valid
+   //
+   if (IX.check_range(get_shape()))   INDEX_ERROR;
+
 MultiIndexIterator mult(get_shape(), IX);   // deletes IDX
 
 Value_P Z(new Value(shape_Z, LOC));
@@ -1031,7 +1051,7 @@ Value::index(Token & IX) const
 
    // not supposed to happen
    //
-   Q(IX.get_tag());  FIXME;
+   Q1(IX.get_tag());  FIXME;
 }
 //-----------------------------------------------------------------------------
 Rank
@@ -1380,25 +1400,25 @@ const Cell & cell_0 = get_ravel(0);
 
    if (cell_0.is_character_cell())   // 8 or 32 bit characters.
       {
-        bool big = false;   // assume 8-bit char
+        bool has_big = false;   // assume 8-bit char
         loop(e, ec)
            {
              const Cell & cell = get_ravel(e);
              if (!cell.is_character_cell())   return 7;
              const Unicode uni = cell.get_char_value();
-             if (uni < 0)      big = true;
-             if (uni >= 256)   big = true;
+             if (uni < 0)      has_big = true;
+             if (uni >= 256)   has_big = true;
            }
 
-        return big ? 5 : 4;   // 8-bit or 32-bit char
+        return has_big ? 5 : 4;   // 8-bit or 32-bit char
       }
 
    if (cell_0.is_numeric())
       {
-        bool is_bool    = false;
-        bool is_int     = false;
-        bool is_float   = false;
-        bool is_complex = false;
+        bool has_bool    = false;
+        bool has_int     = false;
+        bool has_float   = false;
+        bool has_complex = false;
 
         loop(e, ec)
            {
@@ -1406,27 +1426,26 @@ const Cell & cell_0 = get_ravel(0);
              if (cell.is_integer_cell())
                 {
                   const APL_Integer i = cell.get_int_value();
-                  const uint32_t high = i >> 32;
-                  if (i == 0)                    is_bool  = true;
-                  else if (i == 1)               is_bool  = true;
-                  else if (high == 0x00000000)   is_int   = true;
-                  else if (high == 0xFFFFFFFF)   is_int   = true;
-                  else /* too big for int32 */   is_float = true;
+                  if (i == 0)                   has_bool  = true;
+                  else if (i == 1)              has_bool  = true;
+                  else if (i >  0x7FFFFFFFLL)   has_float = true;
+                  else if (i < -0x80000000LL)   has_float = true;
+                  else                          has_int   = true;
                 }
              else if (cell.is_float_cell())
                 {
-                  is_float  = true;
+                  has_float  = true;
                 }
              else if (cell.is_complex_cell())
                 {
-                  is_complex  = true;
+                  has_complex  = true;
                 }
              else return 7;   // mixed: return 7
            }
 
-        if (is_complex)   return 3;
-        if (is_float)     return 2;
-        if (is_int)       return 1;
+        if (has_complex)   return 3;
+        if (has_float)     return 2;
+        if (has_int)       return 1;
         return 0;
       }
 
@@ -1453,7 +1472,7 @@ PrintBuffer pb(*this, pctx);
 
 //   pb.debug(CERR, "Value::print()");
 
-UCS_string ucs(pb, get_rank());
+UCS_string ucs(pb, get_rank(), pctx.get_PW());
 
    if (ucs.size() == 0)   return out;
 
@@ -1469,10 +1488,9 @@ UCS_string ind(indent, UNI_ASCII_SPACE);
        << ind << "Shape:   " << get_shape() << endl
        << ind << "Flags:   " << get_flags();
    if (is_forever())    out << " VF_forever";
-   if (is_left())       out << " VF_left";
-   if (is_dirty())      out << " VF_dirty";
    if (is_complete())   out << " VF_complete";
    if (is_marked())     out << " VF_marked";
+   if (is_temp())       out << " VF_temp";
    out << endl
        << ind << "First:   " << get_ravel(0)  << endl
        << ind << "Dynamic: ";
@@ -1494,7 +1512,7 @@ Value::print_boxed(ostream & out, const char * info)
 {
    if (info)   out << info << endl;
 
-Value_P Z = Quad_CR::fun.do_CR(4, *this);
+Value_P Z = Quad_CR::do_CR(4, *this);
    out << *Z << endl;
    return out;
 }
@@ -1740,8 +1758,6 @@ int count = 0;
          stale_vals.push_back(val);
          stale_dobs.push_back(dob);
 
-         if (!val->is_dirty())   ++count;
-
          if (!goon)
             {
               out << "Value::print_stale() : endless loop in "
@@ -1787,7 +1803,6 @@ int count = 0;
          if (val->is_marked())
             {
               val->print_stale_info(out, dob);
-              if (!val->is_dirty())   ++count;
               val->unmark();
             }
        }
@@ -1806,7 +1821,7 @@ Value::print_stale_info(ostream & out, const DynamicObject * dob)
    try 
       {
         print_structure(out, 0, 0);
-        Value_P Z = Quad_CR::fun.do_CR(7, *this);
+        Value_P Z = Quad_CR::do_CR(7, *this);
         Z->print(out);
         out << endl;
       }
