@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2013  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2014  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "Logging.hh"
 #include "main.hh"
 #include "Output.hh"
+#include "NativeFunction.hh"
 #include "Prefix.hh"
 #include "ProcessorID.hh"
 #include "Quad_SVx.hh"
@@ -53,6 +54,9 @@
 bool silent = false;
 bool emacs_mode = false;
 bool do_not_echo = false;
+bool safe_mode = false;
+
+const char * build_tag[] = { BUILDTAG, 0 };
 
 //-----------------------------------------------------------------------------
 /// initialize subsystems
@@ -76,19 +80,23 @@ cleanup()
 {
    ProcessorID::disconnect();
 
+   NativeFunction::cleanup();
    Input::exit_readline();   // write history
    Output::reset_colors();
 }
 //-----------------------------------------------------------------------------
 
-APL_time interrupt_when = 0;
+APL_time_us interrupt_when = 0;
 bool interrupt_raised = false;
 bool attention_raised = false;
 
+static struct sigaction old_control_C_action;
+static struct sigaction new_control_C_action;
+
 static void
 control_C(int)
 {
-APL_time when = now();
+APL_time_us when = now();
    attention_raised = true;
    if ((when - interrupt_when) < 500000)   // second ^C within 500 ms
       {
@@ -98,12 +106,12 @@ APL_time when = now();
    interrupt_when = when;
 }
 
-static struct sigaction old_control_C_action;
-static struct sigaction new_control_C_action;
-
 //-----------------------------------------------------------------------------
+static struct sigaction old_SEGV_action;
+static struct sigaction new_SEGV_action;
+
 static void
-seg_fault(int)
+signal_SEGV_handler(int)
 {
    CERR << "\n\n====================================================\n"
            "SEGMENTATION FAULT" << endl;
@@ -118,9 +126,6 @@ seg_fault(int)
    Command::cmd_OFF(3);
 }
 
-static struct sigaction old_segfault_action;
-static struct sigaction new_segfault_action;
-
 //-----------------------------------------------------------------------------
 static void
 signal_USR1_handler(int)
@@ -131,6 +136,28 @@ signal_USR1_handler(int)
 static struct sigaction old_USR1_action;
 static struct sigaction new_USR1_action;
 
+//-----------------------------------------------------------------------------
+static struct sigaction old_TERM_action;
+static struct sigaction new_TERM_action;
+
+static void
+signal_TERM_handler(int)
+{
+   cleanup();
+   sigaction(SIGTERM, &old_TERM_action, 0);
+   raise(SIGTERM);
+}
+//-----------------------------------------------------------------------------
+static struct sigaction old_HUP_action;
+static struct sigaction new_HUP_action;
+
+static void
+signal_HUP_handler(int)
+{
+   cleanup();
+   sigaction(SIGHUP, &old_HUP_action, 0);
+   raise(SIGHUP);
+}
 //-----------------------------------------------------------------------------
 /**
     When APL is started as a script. then several options might be grouped
@@ -265,7 +292,6 @@ show_argv(int argc, const char ** argv)
 static void
 show_version(ostream & out)
 {
-const char * build_tag[] = { BUILDTAG, 0 };
    out << "BUILDTAG:" << endl
        << "---------" << endl;
    for (const char ** bt = build_tag; *bt; ++bt)
@@ -360,8 +386,16 @@ show_configure_options()
    "    VF_TRACING_WANTED=yes"
 #else
    "    VF_TRACING_WANTED=no (default)"
+#endif
+   << endl <<
+
+#ifdef VISIBLE_MARKERS_WANTED
+   "    VISIBLE_MARKERS_WANTED=yes"
+#else
+   "    VISIBLE_MARKERS_WANTED=no (default)"
 #endif
    << endl
+
    << endl;
 
 
@@ -406,6 +440,7 @@ _("    -l num               turn log facility num (1-%d) ON\n"), LID_MAX-1);
 "    --cfg                show ./configure options used and exit\n"
 "    --gpl                show license (GPL) and exit\n"
 "    --silent             do not show the welcome message\n"
+"    --safe               safe mode (no shared vars, no native functions)\n"
 "    -s, --script         same as --silent --noCIN --noCONT --noColor -f -\n"
 "    -v, --version        show version information and exit\n"
 "    -T testcases ...     run testcases\n"
@@ -432,7 +467,7 @@ show_GPL(ostream & out)
 "    This program is GNU APL, a free implementation of the\n"
 "    ISO/IEC Standard 13751, \"Programming Language APL, Extended\"\n"
 "\n"
-"    Copyright (C) 2008-2013  Dr. Jürgen Sauermann\n"
+"    Copyright (C) 2008-2014  Dr. Jürgen Sauermann\n"
 "\n"
 "    This program is free software: you can redistribute it and/or modify\n"
 "    it under the terms of the GNU General Public License as published by\n"
@@ -457,7 +492,7 @@ show_welcome(ostream & out, const char * argv0)
 {
 char c1[200];
 char c2[200];
-   snprintf(c1, sizeof(c1), _("Welcome to GNU APL version %s"),PACKAGE_VERSION);
+   snprintf(c1, sizeof(c1), _("Welcome to GNU APL version %s"),build_tag[1]);
    snprintf(c2, sizeof(c2), _("for details run: %s --gpl."), argv0);
 
 const char * lines[] =
@@ -474,7 +509,7 @@ const char * lines[] =
   "",
   c1,
   "",
-  "Copyright (C) 2008-2013  Dr. Jürgen Sauermann",
+  "Copyright (C) 2008-2014  Dr. Jürgen Sauermann",
   "Banner by FIGlet: www.figlet.org",
   "",
 _("This program comes with ABSOLUTELY NO WARRANTY;"),
@@ -512,7 +547,6 @@ const int left_pad = (80 - len)/2;
 extern char * bindtextdomain (const char * domainname, const char * dirname);
 extern char * textdomain (const char * domainname);
 
-
 static void
 init_NLS()
 {
@@ -533,6 +567,11 @@ const char * locale = setlocale(LC_ALL, "");
 
    Log(LOG_startup)   CERR << "locale is: " << locale << "." << endl;
 
+   // ensure that numbers are scanned and printed in
+   // a defined APL-compatible manner, regardless of the locale
+   //
+   setlocale(LC_NUMERIC, "C");
+
 const char * dir = bindtextdomain(PACKAGE, LOCALEDIR);
 
    if (dir)
@@ -604,6 +643,9 @@ struct user_preferences
 
    /// randomize the order of testfiles
    bool randomize_testfiles;
+
+   /// safe mode
+   bool safe_mode;
 };
 //-----------------------------------------------------------------------------
 // read user preference file(s) if present
@@ -664,12 +706,28 @@ int line = 0;
          if (*s == '#')   continue;   // comment line
          char opt[BUFSIZE] = { 0 };
          char arg[BUFSIZE] = { 0 };
-         int d[20];
+         int d[100];
          const int count = sscanf(s, "%s"
                    " %s %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
+                   " %X %X %X %X %X %X %X %X %X %X"
                    " %X %X %X %X %X %X %X %X %X %X",
               opt, arg,  d+ 1, d+ 2, d+ 3, d+ 4, d+ 5, d+ 6, d+ 7, d+ 8, d+ 9,
-                   d+10, d+11, d+12, d+13, d+14, d+15, d+16, d+17, d+18, d+19);
+                   d+10, d+11, d+12, d+13, d+14, d+15, d+16, d+17, d+18, d+19,
+                   d+20, d+21, d+22, d+23, d+24, d+25, d+26, d+27, d+28, d+29,
+                   d+30, d+31, d+32, d+33, d+34, d+35, d+36, d+37, d+38, d+39,
+                   d+40, d+41, d+42, d+43, d+44, d+45, d+46, d+47, d+48, d+49,
+                   d+50, d+51, d+52, d+53, d+54, d+55, d+56, d+57, d+58, d+59,
+                   d+60, d+61, d+62, d+63, d+64, d+65, d+66, d+67, d+68, d+69,
+                   d+70, d+71, d+72, d+73, d+74, d+75, d+76, d+77, d+78, d+79,
+                   d+80, d+81, d+82, d+83, d+84, d+85, d+86, d+87, d+88, d+89,
+                   d+90, d+91, d+92, d+93, d+94, d+95, d+96, d+97, d+98, d+99);
 
          if (count < 2)
             {
@@ -800,6 +858,14 @@ int line = 0;
                                     sys ? LibPaths::LibDir::CS_PREF_SYS
                                         : LibPaths::LibDir::CS_PREF_HOME);
             }
+         else if (!strcasecmp(opt, "READLINE_HISTORY_LEN"))
+            {
+              Input::readline_history_len = atoi(arg);
+            }
+         else if (!strcasecmp(opt, "READLINE_HISTORY_PATH"))
+            {
+              Input::readline_history_path = UTF8_string(arg);
+            }
        }
 
    fclose(f);
@@ -827,15 +893,21 @@ user_preferences up;
    //
    memset(&new_control_C_action, 0, sizeof(struct sigaction));
    memset(&new_USR1_action,      0, sizeof(struct sigaction));
-   memset(&new_segfault_action,  0, sizeof(struct sigaction));
+   memset(&new_SEGV_action,      0, sizeof(struct sigaction));
+   memset(&new_TERM_action,      0, sizeof(struct sigaction));
+   memset(&new_HUP_action,       0, sizeof(struct sigaction));
 
    new_control_C_action.sa_handler = &control_C;
-   new_USR1_action     .sa_handler = &signal_USR1_handler;
-   new_segfault_action .sa_handler = &seg_fault;
+   new_USR1_action .sa_handler = &signal_USR1_handler;
+   new_SEGV_action .sa_handler = &signal_SEGV_handler;
+   new_TERM_action .sa_handler = &signal_TERM_handler;
+   new_HUP_action  .sa_handler = &signal_HUP_handler;
 
    sigaction(SIGINT,  &new_control_C_action, &old_control_C_action);
    sigaction(SIGUSR1, &new_USR1_action,      &old_USR1_action);
-   sigaction(SIGSEGV, &new_segfault_action,  &old_segfault_action);
+   sigaction(SIGSEGV, &new_SEGV_action,      &old_SEGV_action);
+   sigaction(SIGTERM, &new_TERM_action,      &old_TERM_action);
+   sigaction(SIGHUP,  &new_HUP_action,       &old_HUP_action);
 
    // init NLS so that usage() will be translated
    //
@@ -961,6 +1033,11 @@ user_preferences up;
             {
               silent = true;
             }
+         else if (!strcmp(opt, "--safe"))
+            {
+              safe_mode = true;
+              up.do_sv = false;
+            }
          else if (!strcmp(opt, "-T"))
             {
               up.do_CONT = false;
@@ -1074,12 +1151,22 @@ user_preferences up;
    // init input after reading the command line options, so that the user
    // has a chance to disable readline
    //
-   Input::init();
+   Input::init(true);
 
    if (up.daemon)
       {
-        if (fork())   return 0;   // parent returns
-        Log(LOG_startup)   CERR << "process forked" << endl;
+        const pid_t pid = fork();
+        if (pid)   // parent
+           {
+             Log(LOG_startup)
+                CERR << "parent pid = " << getpid()
+                     << " child pid = " << pid << endl;
+
+             exit(0);   // parent returns
+           }
+
+        Log(LOG_startup)
+           CERR << "child forked (pid" << getpid() << ")" << endl;
       }
 
    if (up.wait_ms)   usleep(1000*up.wait_ms);
@@ -1102,9 +1189,9 @@ user_preferences up;
 
    if (up.do_CONT)
       {
-         UCS_string cont("CONTINUE.xml");
+         UCS_string cont("CONTINUE");
          UTF8_string filename =
-            LibPaths::get_lib_filename(LIB0, "CONTINUE", true, "xml");
+            LibPaths::get_lib_filename(LIB0, cont, true, "xml");
 
          if (access(filename.c_str(), F_OK) == 0)
             {
