@@ -108,7 +108,7 @@ Value_P format = A->get_ravel(a++).get_pointer_value();
          char fmt[40];
          int fm = 0;   // an index into fmt;
          fmt[fm++] = '%';
-         for (; fm < (sizeof(fmt) - 1); ++fm)
+         for (; fm < (sizeof(fmt) - 1); fm)
              {
                if (f >= format->element_count())
                   {
@@ -143,7 +143,7 @@ Value_P format = A->get_ravel(a++).get_pointer_value();
                          case 'q':
                          case 'j':
                          case 'z':
-                         case 't': fmt[fm++] = un1;
+                         case 't': fmt[fm++] = un1;   fmt[fm] = 0;
                                    continue;
 
                          // conversion specifier
@@ -151,10 +151,19 @@ Value_P format = A->get_ravel(a++).get_pointer_value();
                          case 'd':   case 'i':   case 'o':
                          case 'u':   case 'x':   case 'X':   case 'p':
                               {
-                                const APL_Integer iv =
-                                            A->get_ravel(a++).get_int_value();
-                                fmt[fm++] = un1;
-                                fmt[fm] = 0;
+                                const Cell & cell = A->get_ravel(a++);
+                                APL_Integer iv;
+                                if (cell.is_integer_cell())
+                                   {
+                                     iv = cell.get_int_value();
+                                   }
+                                else 
+                                   {
+                                     const APL_Float fv = cell.get_real_value();
+                                     if (fv < 0)   iv = -int(-fv);
+                                     else          iv = int(fv);
+                                   }
+                                fmt[fm++] = un1;   fmt[fm] = 0;
                                 out_len += fprintf(out, fmt, iv);
                               }
                               goto field_done;
@@ -164,8 +173,7 @@ Value_P format = A->get_ravel(a++).get_pointer_value();
                               {
                                 const APL_Float fv =
                                             A->get_ravel(a++).get_real_value();
-                                fmt[fm++] = un1;
-                                fmt[fm] = 0;
+                                fmt[fm++] = un1;   fmt[fm] = 0;
                                 out_len += fprintf(out, fmt, fv);
                               }
                               goto field_done;
@@ -311,6 +319,7 @@ list_functions(ostream & out)
 "   Zi ← A  FUN[22] 1     printf(         A1, A2...) format A1\n"
 "   Zi ← A  FUN[22] 2     fprintf(stderr, A1, A2...) format A1\n"
 "   Zi ← A  FUN[22] Bh    fprintf(Bh,     A1, A2...) format A1\n"
+"   Zi ← Ac FUN[23] Bh    fwrite(Ac, 1, ⍴Ac, Bh) Unicode Ac Output UTF-8\n"
 "\n";
 
    return Token(TOK_APL_VALUE1, Value::Str0_P);
@@ -594,7 +603,7 @@ const int function_number = X->get_ravel(0).get_near_int(qct);
                 Value_P Z(new Value(len, LOC));
                 new (&Z->get_ravel(0)) IntCell(0);   // prototype
                 loop(z, len)   new (&Z->get_ravel(z)) IntCell(buffer[z] & 0xFF);
-                delete del;
+                delete [] del;
                 Z->check_value(LOC);
                 return Token(TOK_APL_VALUE1, Z);
               }
@@ -612,9 +621,10 @@ const int function_number = X->get_ravel(0).get_near_int(qct);
                    buffer = del = new char[bytes];
 
                 loop(z, bytes)   buffer[z] = A->get_ravel(z).get_near_int(qct);
-                delete del;
 
                 size_t len = fwrite(buffer, 1, bytes, fe.fe_file);
+                delete [] del;
+
                 Value_P Z(new Value(LOC));
                 new (&Z->get_ravel(0)) IntCell(len); 
                 Z->check_value(LOC);
@@ -639,7 +649,7 @@ const int function_number = X->get_ravel(0).get_near_int(qct);
                 Value_P Z(new Value(len, LOC));
                 new (&Z->get_ravel(0)) IntCell(0);   // prototype
                 loop(z, len)   new (&Z->get_ravel(z)) IntCell(buffer[z] & 0xFF);
-                delete del;
+                delete [] del;
                 Z->check_value(LOC);
                 return Token(TOK_APL_VALUE1, Z);
               }
@@ -685,6 +695,19 @@ const int function_number = X->get_ravel(0).get_near_int(qct);
                 file_entry & fe = get_file(*B.get());
                 return do_printf(fe.fe_file, A);
               }
+
+         case 23:   // fwrite(Ac, 1, ⍴Ac, Bh) Unicode Ac Output UTF-8 \n"
+              {
+                errno = 0;
+                file_entry & fe = get_file(*B.get());
+                UCS_string text(*A.get());
+                UTF8_string utf(text);
+                const size_t l = fwrite(utf.c_str(), 1, utf.size(), fe.fe_file); 
+                Value_P Z(new Value(LOC));
+                new (&Z->get_ravel(0)) IntCell(l); 
+                Z->check_value(LOC);
+                return Token(TOK_APL_VALUE1, Z);
+              }
       }
 
    CERR << "eval_AXB() function number: " << function_number << endl;
